#!/usr/bin/perl
#$Id: account_config.cgi,v 1.2 2002/12/12 22:09:46 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# account_config.cgi  -  Created by James Pattie, 05/20/1999
# This script allows the user to configure the Accounting package
# in particular the accounts.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

use vars '$date';
my $ledger_db;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print "$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# see if there are any ledgers to work with.
$result = $conn->exec("SELECT * FROM ledger_tb");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Checking for Ledgers failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}
my $ledgers_exist;
$ledgers_exist = $result->ntuples;

# present them with the main menu

print "<head><title>$prog_name $prog_version Program Configuration (Accounts)</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><br></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#accounting_config\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Account Configuration</h2>\n";

if (length $ledger_db > 0)
{
  print "<form method=post action=\"\" name=\"account_form\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<input type=hidden name=\"select_ledger\" value=\"\">\n";
}

# now we give them the menu options for account configuration
if (length $ledger_db == 0)
{
  print "<br><br><div align=center><a href=\"$site_dir/cgi-bin/account_options.cgi\">Account Options</a></div>\n";
  print "<br><div align=center><a href=\"$site_dir/cgi-bin/create_account.cgi\">Create new Account</a></div>\n";
}
else
{
  print "<br><br><div align=center><a href=\"$site_dir/cgi-bin/account_options.cgi\" onClick=\"go_here(document.account_form, '$site_dir/cgi-bin/account_options.cgi'); return false;\">Account Options</a></div>\n";
  print "<br><div align=center><a href=\"$site_dir/cgi-bin/create_account.cgi\" onClick=\"go_here(document.account_form, '$site_dir/cgi-bin/create_account.cgi'); return false;\">Create new Account</a></div>\n";
}

# check and see if we have any accounts that the user can work with for either editing or assigning to Ledgers
$result = $conn->exec("select * from account_tb");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print "Tried to get count of accounts available.<br>$sys_admin";
  CleanUp();
}
if (length $ledger_db == 0)
{
  if ($result->ntuples > 0)  # only do this if accounts are created.
  {
    print "<br><div align=center><a href=\"$site_dir/cgi-bin/edit_account.cgi\">Edit Account</a></div>\n";
    # see if there are any ledgers available to work with
    $result = $conn->exec("select * from entity_tb");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      print "<h1>Error:  Accessing database failed!</h1>\n";
      print "Tried to get count of Ledgers available.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples > 0 && $ledgers_exist)
    {
      print "<br><div align=center><a href=\"$site_dir/cgi-bin/assign_accounts.cgi\">Assign Accounts to Ledgers</a></div>\n";
    }
  }
}
else
{
  if ($result->ntuples > 0) # only do this if there are accounts defined
  {
    print "<br><div align=center><a href=\"$site_dir/cgi-bin/edit_account.cgi\" onClick=\"go_here(document.account_form, '$site_dir/cgi-bin/edit_account.cgi'); return false;\">Edit Account</a></div>\n";
    # see if there are any ledgers available to work with
    $result = $conn->exec("select * from entity_tb");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      print "<h1>Error:  Accessing database failed!</h1>\n";
      print "Tried to get count of Ledgers available.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples > 0 && $ledgers_exist)
    {
      print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";  # used to signal that I am really inside of a ledger and to provide the return to ledger link.
      print "<br><div align=center><a href=\"$site_dir/cgi-bin/assign_accounts2.cgi\" onClick=\"go_here(document.account_form, '$site_dir/cgi-bin/assign_accounts2.cgi'); return false;\">Assign Accounts to Ledgers</a></div>\n";
    }
  }
}

if (length $ledger_db > 0)
{
  print "</form>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;
