#!/usr/bin/perl
#$Id: account_options2.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# account_options2.cgi  -  Created by James Pattie, 01/24/2000
# This script allows the user to configure the Accounting package
# and specify the account specific options.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features

use vars '$date';
my $ledger_db;
my $acct_length;
my $sub_acct_length;
my $error_string;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};

if (not exists $input{'update'})
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "Invalid options provided!<br>$sys_admin";
  CleanUp();
}

$acct_length = $input{'acct_length'};
if (length $acct_length == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the Number of Digits that make up an Account!</h1>\n";
  print "$back_button";
  CleanUp();
}
$error_string = IsValidInteger($acct_length);
if (length $error_string != 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Account Length!</h1>\n";
  print "<b>$error_string</b> is invalid.<br>$back_button";
  CleanUp();
}
if ($acct_length <= 2 || $acct_length > 10)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Value for the Account Length!</h1>\n";
  print "<b>$acct_length</b> is invalid.<br>$back_button";
  CleanUp();
}

$sub_acct_length = $input{'sub_acct_length'};
if (length $sub_acct_length == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the Number of Digits that make up a Sub Account!</h1>\n";
  print "$back_button";
  CleanUp();
}
$error_string = IsValidInteger($sub_acct_length);
if (length $error_string != 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Sub Account Length!</h1>\n";
  print "<b>$error_string</b> is invalid.<br>$back_button";
  CleanUp();
}
if ($sub_acct_length <= 3 || $sub_acct_length > 10)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Value for the Sub Account Length!</h1>\n";
  print "<b>$sub_acct_length</b> is invalid.<br>$back_button";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# first verify that there are no accounts or sub-accounts created that will now be considered too long.
$result = $conn->exec("SELECT max(number) FROM account_tb");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Getting the biggest Account failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}
if (length $result->getvalue(0, 0) > $acct_length)
{
  PrintErrorHeader();
  print "<h1>Error:  Account Length is too short!</h1>\n";
  print "There is at least 1 Account that will be too long!<br>$back_button";
  CleanUp();
}

$result = $conn->exec("SELECT max(sub_acc_number) FROM sub_acc_tb");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Getting the biggest Sub Account failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}
if (length $result->getvalue(0, 0) > $acct_length)
{
  PrintErrorHeader();
  print "<h1>Error:  Sub Account Length is too short!</h1>\n";
  print "There is at least 1 Sub Account that will be too long!<br>$back_button";
  CleanUp();
}

# update the database.
$result = $conn->exec("BEGIN WORK");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  print "<h1>Error:  Starting the Transaction failed!</h1>\n";
  print $conn->errorMessage . "<br>Before Account Option update.<br>$sys_admin";
  CleanUp();
}

$result = $conn->exec("UPDATE version_tb SET acc_digits = $acct_length, sub_acc_digits = $sub_acct_length");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  print "<h1>Error:  Updating the Account Options failed!</h1>\n";
  print $conn->errorMessage . "<br>Account Length = '$acct_length', Sub Account Length = '$sub_acct_length'.<br>$sys_admin";
  CleanUp();
}

$result = $conn->exec("COMMIT WORK");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  print "<h1>Error:  Committing the Transaction failed!</h1>\n";
  print $conn->errorMessage . "<br>After Account Option update.<br>$sys_admin";
  CleanUp();
}


# present them with the main menu

print "<head><title>$prog_name $prog_version Account Options (Update)</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><br></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#account_options\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Account Options (Update)</h2>\n";

print "<br><br><h2>Account Options Successfully Updated!</h2><br>\n";
print "<form method=post action=\"$site_dir/cgi-bin/account_options.cgi\" name=\"account_form\">\n";
if (length $ledger_db > 0)
{
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
}
print "<input type=hidden name=\"nothing\" value=\"\">\n";
print "<a href=\"$site_dir/cgi-bin/account_options.cgi\" onClick=\"document.account_form.submit(); return false;\">To Re-edit the Account Options, go <u>here</u>.</a>\n";
print "</form>\n";

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

if (length $ledger_db == 0)
{
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\">Program Configuration</a></div>\n";
}
else
{
  print "<form name=\"form1\" method=post action=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/account_config.cgi'); return false;\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/prg_config.cgi'); return false;\">Program Configuration</a></div>\n";
  print "<input type=hidden name=\"open\" value=\"Open\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/ledger.cgi'); return false;\">Return to Ledger Summary</a></div>\n";
  print "</form>\n";
}
print "</body>\n";
print "</html>\n";
exit 0;
