#!/usr/bin/perl
#$Id: assign_accounts3.cgi,v 1.3 2002/01/29 16:16:54 pcxuser Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# assign_accounts3.cgi  -  Created by James Pattie, 05/25/1999
# This script takes either the ledger or account the user selected
# and update the database according to what they selected.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
my $report_obj;
my $ledger_code;
my $really_in_ledger;
my $assigned_accounts;
my $unassigned_accounts;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$really_in_ledger = $input{'really_in_ledger'};

# validate we have correct data
my ($mode, $acct_num, $acct_name, $ledger_db, $ledger_name, $ledger_db_name, $result2, $result3);
my ($ledger_num, $account_num);
my ($error_string, $error_occurred);
$error_string = "";
$error_occurred = 0;
# result2, result3 will be used to have multiple select query results stored in.
# conn2 will be used to connect to each of the ledger databases as needed.
# modes are 1=accounts, 2=ledgers
if (not (exists $input{'submit_account'} or exists $input{'submit_ledger'}))
{
  PrintErrorHeader();
  print "<h1>Error:  Access denied!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
if (exists $input{'submit_account'})
{
  $mode = 1;
  $acct_name = $input{'acct_name'};
  if (length $acct_name == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  You must specify a valid account to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $acct_num = $input{'acct_num'};
  if (length $acct_num == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  You must select an account to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $ledger_num = $input{'ledger_num'};
  if (length $ledger_num == 0 || $ledger_num <= 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid  # of ledger's to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }
}
elsif (exists $input{'submit_ledger'})
{
  $mode = 2;
  $ledger_db = $input{'ledger'};
  if (length $ledger_db == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  You must select a ledger to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $ledger_name = $input{'ledger_name'};
  if (length $ledger_name == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  You must specify a ledger to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }

  $account_num = $input{'account_num'};
  if (length $account_num == 0 || $account_num <= 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid # of accounts to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }

  if (!exists $input{'assigned_accounts'})
  {
    PrintErrorHeader();
    print "<h1>Error:  The assigned accounts string was not defined!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  if (!exists $input{'unassigned_accounts'})
  {
    PrintErrorHeader();
    print "<h1>Error:  The un-assigned accounts string was not defined!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  $assigned_accounts = $input{'assigned_accounts'};
  $unassigned_accounts = $input{'unassigned_accounts'};
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# validate we got a valid account or ledger based upon the mode we are in.
if ($mode == 1)
{
  $result = $conn->exec("select active from account_tb where number = '$acct_num' and name = '$acct_name'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get list of accounts.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    PrintErrorHeader();
    print "<h1>Internal Error:  The account <font color=\"blue\">$acct_name</font> does not exist!</h1>\n";
    print "Tried to validate account <font color=\"blue\">$acct_name</font>.<br>$sys_admin";
    CleanUp();
  }
  else
  {
    my $active = $result->getvalue(0, 0);
    if ($active eq "f")
    {
      PrintErrorHeader();
      print "<h1>Error:  The account <font color=\"blue\">$acct_name</font> is not active!</h1>\n";
      print "Tried to validate account <font color=\"blue\">$acct_name</font>.<br>$sys_admin";
      CleanUp();
    }
  }
}
elsif ($mode == 2)
{
  $result = $conn->exec("select database_name from entity_tb where name = '$ledger_name' and database_name = '$ledger_db'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database failed!</h1><br>\n";
    print $conn->errorMessage . "<br>Tried to validate ledger <font color=\"blue\">$ledger_name</font>.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    PrintErrorHeader();
    print "<h1>Internal Error:  The ledger <font color=\"blue\">$ledger_name</font> does not exist!</h1>\n";
    print "Tried to validate ledger <font color=\"blue\">$ledger_name</font>.<br>$sys_admin";
    CleanUp();
  }
}

# now we get all the data being passed in and add it to the database(s) and update stuff.
if ($mode == 1)
{
  my $i;
  for ($i=0; $i < $ledger_num; $i++)
  {
    my ($name, $status, $temp, $temp2);
    $temp = "ledger_" . "$i";
    if (not exists $input{$temp})
    {
      PrintErrorHeader();
      print "<h1>Internal Error:  Ledger entry <font color=\"blue\"># $i</font> does not exist!</h1>\n";
      print "Tried to validate ledger <font color=\"blue\"># $i</font>.<br>$sys_admin";
      CleanUp();
    }
    $temp2 = "radio_" . "$i";
    if (not exists $input{$temp2})
    {
      PrintErrorHeader();
      print "<h1>Internal Error:  Ledger entry status <font color=\"blue\"># $i</font> does not exist!</h1>\n";
      print "Tried to validate ledger status <font color=\"blue\"># $i</font>.<br>$sys_admin";
      CleanUp();
    }
    $name = $input{$temp};
    $status = $input{$temp2};
    if ($status eq "YES")
    {
      $status = 1;
    }
    else
    {
      $status = 0;
    }

    # now we do the database updates.
    # first we need to lookup the ledger name, validate it and get the database name for the ledger.
    $result = $conn->exec("select database_name from entity_tb where name = '$name'");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Internal Error:  Accessing database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to validate ledger <font color=\"blue\">$name</font>.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples == 0)
    {
      PrintErrorHeader();
      print "<h1>Internal Error:  Ledger <font color=\"blue\">$name</font> does not exist!</h1>\n";
      print "Tried to validate ledger <font color=\"blue\">$name</font>.<br>$sys_admin";
      CleanUp();
    }
    $ledger_db_name = $result->getvalue(0, 0);

    # create the reports object
    $report_obj = reports->new($conn, $ledger_db_name);
    if ($report_obj->didErrorOccur())
    {
      PrintErrorHeader();
      print $report_obj->getErrorMessage();
      CleanUp();
    }
    $ledger_code = $report_obj->getLedgerCode();
    if ($report_obj->didErrorOccur())
    {
      PrintErrorHeader();
      print $report_obj->getErrorMessage();
      CleanUp();
    }
    # now we either create the entry for the account if it needs to be set, or if it was set and is now being turned off,
    # then we remove all references to it from the database.
    $result = $conn->exec("begin work");
    if ($result->resultStatus != PGRES_COMMAND_OK)
    {
       PrintErrorHeader();
       print "<h1>Error:  Couldn't start a transaction with database <font color=\"blue\">$ledger_db_name</font>!</h1>\n";
       print $conn->errorMessage . "<br>$sys_admin";
       CleanUp();
    }

    if ($status == 1) # they want it turned on
    {
      # first we need to check and see if it exists.
      $result = $conn->exec("select * from local_accounts_tb where acct_num = $acct_num and sub_entity_code is null AND ledger_code = '$ledger_code'");
      if ($result->resultStatus != PGRES_TUPLES_OK)
      {
        PrintErrorHeader();
        print "<h1>Error:  Searching database <font color=\"blue\">$ledger_db_name</font> for account <font color=\"blue\">$acct_num</font> failed!</h1>\n";
        print "Tried to find account <font color=\"blue\">$acct_num</font> in ledger <font color=\"blue\">$name</font>.<br>\n";
        print $conn->errorMessage . "<br>$sys_admin";
        $result = $conn->exec("ABORT TRANSACTION");
        CleanUp();
      }
      if ($result->ntuples == 0)
      { # insert, else do nothing  (Changed default tax status from false to true, but only if an Expense or Revenue Account)
        $result = $conn->exec("SELECT taxable FROM account_tb WHERE number = $acct_num");
        if ($result->resultStatus != PGRES_TUPLES_OK)
        {
          PrintErrorHeader();
          print "<h1>Error:  Accessing database failed!</h1>\n" . $conn->errorMessage . "<br>Tried to get Taxable Value for Account = '$acct_num'.<br>$sys_admin";
          $result = $conn->exec("ABORT TRANSACTION");
          CleanUp();
        }
        if ($result->ntuples == 0)
        {
          PrintErrorHeader();
          print "<h1>Error:  Account not found!</h1>\nAccount = '$acct_num' does not appear to exist anymore!<br>$sys_admin";
          $result = $conn->exec("ABORT TRANSACTION");
          CleanUp();
        }
        my $acct_taxable = $result->getvalue(0,0);

        $result = $conn->exec("insert into local_accounts_tb (acct_num, taxable, ledger_code) values ($acct_num, '$acct_taxable', '$ledger_code')");
        if ($result->resultStatus != PGRES_COMMAND_OK)
        {
          PrintErrorHeader();
          print "<h1>Error:  Inserting data into database <font color=\"blue\">$ledger_db_name</font> failed!</h1>\n";
          print "Tried to insert account <font color=\"blue\">$acct_num</font> into ledger <font color=\"blue\">$name</font>.<br>\n";
          print $conn->errorMessage . "<br>$sys_admin";
          $result = $conn->exec("abort transaction");
          CleanUp();
        }
      }
    }
    elsif ($status == 0) # they want it turned off if it was on
    {
      # check and see if it is turned on.
      $result = $conn->exec("select * from local_accounts_tb where acct_num = $acct_num AND ledger_code = '$ledger_code'");
      if ($result->resultStatus != PGRES_TUPLES_OK)
      {
        PrintErrorHeader();
        print "<h1>Error:  Searching database <font color=\"blue\">$ledger_db_name</font> for account <font color=\"blue\">$acct_num</font> failed!</h1>\n";
        print "Tried to find account <font color=\"blue\">$acct_num</font> in ledger <font color=\"blue\">$name</font>.<br>\n";
        print $conn->errorMessage . "<br>$sys_admin";
        $result = $conn->exec("ABORT TRANSACTION");
        CleanUp();
      }
      if ($result->ntuples > 0)
      {
        # verify that this account is not in use!
        $result = $conn->exec("SELECT * FROM debitcredit_tb WHERE account_num = $acct_num AND ledger_code = '$ledger_code'");
        if ($result->resultStatus != PGRES_TUPLES_OK)
        {
          PrintErrorHeader();
          print "<h1>Error:  Accessing database failed!</h1>\n";
          print $conn->errorMessage . "<br>Tried to see if Account = $acct_num is being used in ledger '$name'.<br>$sys_admin";
          CleanUp();
        }
        if ($result->ntuples > 0)
        {
          $error_string .= "Account = $acct_num is being used in ledger '$name'!  Skipping...<br>\n";
          $error_occurred = 1;
        }
        else
        {
          # we have to delete them!
          $result = $conn->exec("delete from local_accounts_tb where acct_num = $acct_num AND ledger_code = '$ledger_code'");
          if ($result->resultStatus != PGRES_COMMAND_OK)
          {
            PrintErrorHeader();
            print "<h1>Error:  Deleting account <font color=\"blue\">$acct_num</font> failed!</h1>\n";
            print "Tried to delete account <font color=\"blue\">$acct_num</font> from ledger <font color=\"blue\">$name</font>.<br>\n";
            print $conn->errorMessage . "<br>$sys_admin";
            $result = $conn->exec("ABORT TRANSACTION");
            CleanUp();
          }
        }
      }
    }

    # commit the work
    $result = $conn->exec("commit work");
    if ($result->resultStatus != PGRES_COMMAND_OK)
    {
       PrintErrorHeader();
       print "<h1>Error:  Couldn't commit the transaction to database <font color=\"blue\">$ledger_db_name</font>!</h1>\n";
       print $conn->errorMessage . "<br>$sys_admin";
       CleanUp();
    }
  }
}
elsif ($mode == 2)
{
  # create the reports object
  $report_obj = reports->new($conn, $ledger_name);
  if ($report_obj->didErrorOccur())
  {
    PrintErrorHeader();
    print $report_obj->getErrorMessage();
    CleanUp();
  }
  $ledger_code = $report_obj->getLedgerCode();
  if ($report_obj->didErrorOccur())
  {
    PrintErrorHeader();
    print $report_obj->getErrorMessage();
    CleanUp();
  }

  # start a transaction
  $result = $conn->exec("begin work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't start a transaction in database <font color=\"blue\">$ledger_name</font>!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  my @assigned_accounts = split /\|/, $assigned_accounts;
  my @unassigned_accounts = split /\|/, $unassigned_accounts;

  my $i;
  for ($i=0; $i < scalar @assigned_accounts; $i++)
  {
    my ($name, $num) = split /!!/, $assigned_accounts[$i];

    # first we validate that this account is valid and active.
    $result = $conn->exec("select active from account_tb where number = '$num'");# and name = '$name'");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error:  Accessing database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to validate account <font color=\"blue\">$name</font>.<br>\nIn Assign Account Loop.\n<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples == 0)
    {
      PrintErrorHeader();
      print "<h1>Internal Error:  Account entry <font color=\"blue\">$name - $num</font> does not exist!</h1>\n";
      print "In Assign Account Loop.<br>$sys_admin";
      CleanUp();
    }
    else
    {
      my $active = $result->getvalue(0, 0);
      if ($active eq "f")
      {
        PrintErrorHeader();
        print "<h1>Error:  Account <font color=\"blue\">$name</font> - <font color=\"blue\">$num</font> is not active!</h1>\nIn Assign Account Loop.\n<br>";
        print "$sys_admin";
        CleanUp();
      }
    }

    # first we need to check and see if it exists.
    $result = $conn->exec("select * from local_accounts_tb where acct_num = $num and sub_entity_code is null AND ledger_code = '$ledger_code'");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error:  Searching database <font color=\"blue\">$ledger_db</font> for account <font color=\"blue\">$num</font> failed!</h1>\n";
      print "Tried to find account <font color=\"blue\">$name</font> in ledger <font color=\"blue\">$ledger_name</font>.<br>\nIn Assign Account Loop.\n<br>";
      print $conn->errorMessage . "<br>Mode = 2<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples == 0)
    { # insert, else do nothing  (Changed default tax status from false to true)
      $result = $conn->exec("SELECT taxable FROM account_tb WHERE number = $num");
      if ($result->resultStatus != PGRES_TUPLES_OK)
      {
        PrintErrorHeader();
        print "<h1>Error:  Accessing database failed!</h1>\n" . $conn->errorMessage . "<br>Tried to get Taxable Value for Account = '$num'.<br>\nIn Assign Account Loop.\n<br>$sys_admin";
        $result = $conn->exec("ABORT TRANSACTION");
        CleanUp();
      }
      if ($result->ntuples == 0)
      {
        PrintErrorHeader();
        print "<h1>Error:  Account not found!</h1>\nAccount = '$num' does not appear to exist anymore!<br>\nIn Assign Account Loop.\n<br>$sys_admin";
        $result = $conn->exec("ABORT TRANSACTION");
        CleanUp();
      }
      my $acct_taxable = $result->getvalue(0,0);

      $result = $conn->exec("insert into local_accounts_tb (acct_num, taxable, ledger_code) values ($num, '$acct_taxable', '$ledger_code')");
      if ($result->resultStatus != PGRES_COMMAND_OK)
      {
        PrintErrorHeader();
        print "<h1>Error:  Inserting data into database <font color=\"blue\">$ledger_name</font> failed!</h1>\n";
        print "Tried to insert account <font color=\"blue\">$name</font> into ledger <font color=\"blue\">$ledger_name</font>.<br>\n";
        print $conn->errorMessage . "<br>\nIn Assign Account Loop.\n<br>$sys_admin";
        $result = $conn->exec("abort transaction");
        CleanUp();
      }
    }
  }

  for ($i=0; $i < scalar @unassigned_accounts; $i++)
  {
    my ($name, $num) = split /!!/, $unassigned_accounts[$i];

    # first we validate that this account is valid and active.
    $result = $conn->exec("select active from account_tb where number = '$num'"); # and name = '$name'");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error:  Accessing database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to validate account <font color=\"blue\">$name</font>.<br>\nIn Un-Assign Account Loop.\n<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples == 0)
    {
      PrintErrorHeader();
      print "<h1>Internal Error:  Account entry <font color=\"blue\">$name - $num</font> does not exist!</h1>\n";
      print "In Un-Assign Account Loop.<br>$sys_admin";
      CleanUp();
    }
    else
    {
      my $active = $result->getvalue(0, 0);
      if ($active eq "f")
      {
        PrintErrorHeader();
        print "<h1>Error:  Account <font color=\"blue\">$name</font> - <font color=\"blue\">$num</font> is not active!</h1>\nIn Un-Assign Account Loop.\n<br>";
        print "$sys_admin";
        CleanUp();
      }
    }

    # check and see if it is turned on.
    $result = $conn->exec("select * from local_accounts_tb where acct_num = $num AND ledger_code = '$ledger_code'");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error:  Searching database <font color=\"blue\">$ledger_name</font> for account <font color=\"blue\">$num</font> failed!</h1>\n";
      print "Tried to find account <font color=\"blue\">$name</font> in ledger <font color=\"blue\">$ledger_name</font>.<br>\n";
      print $conn->errorMessage . "<br>\nIn Un-Assign Account Loop.\n<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples > 0)
    {
      # verify that this account is not in use!
      $result = $conn->exec("SELECT * FROM debitcredit_tb WHERE account_num = $num AND ledger_code = '$ledger_code'");
      if ($result->resultStatus != PGRES_TUPLES_OK)
      {
        PrintErrorHeader();
        print "<h1>Error:  Accessing database failed!</h1>\n";
        print $conn->errorMessage . "<br>Tried to see if Account ($name) = $num is being used in ledger '$ledger_name'.<br>\nIn Un-Assign Account Loop.\n<br>$sys_admin";
        CleanUp();
      }
      if ($result->ntuples > 0)
      {
        $error_string .= "Account ($name) = $num is being used in ledger '$ledger_name'!  Skipping...<br>\n";
        $error_occurred = 1;
      }
      else
      {
        # we have to delete them!
        $result = $conn->exec("delete from local_accounts_tb where acct_num = $num AND ledger_code = '$ledger_code'");
        if ($result->resultStatus != PGRES_COMMAND_OK)
        {
          PrintErrorHeader();
          print "<h1>Error:  Deleting account <font color=\"blue\">$name</font> failed!</h1>\n";
          print "Tried to delete account <font color=\"blue\">$name</font> from ledger <font color=\"blue\">$ledger_name</font>.<br>\n";
          print $conn->errorMessage . "<br>\nIn Un-Assign Account Loop.\n<br>$sys_admin";
          CleanUp();
        }
      }
    }
  }

  # now we commit our work
  $result = $conn->exec("commit work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Committing work failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

print "<head><title>$prog_name $prog_version Ledger Configuration (Accounts)</title>\n";
print <<"END_OF_STYLE";
<script language="Javascript1.2">
<!--
  function SubmitForm()
  { // This function will submit the assign_account_form with the needed stuff set.
    document.assign_account_form.submit();
  }
//-->
</script>
END_OF_STYLE
print "</head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!- -
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\" width=50%>\n";
print "<tr>\n";
print "<td width=47%><br></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#ledger_account_update\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Ledger Account Database Assignment</h2>\n";

if ($error_occurred)
{
  print "$error_string<br>\n";
}

print "Ledger updated successfully!<br><br>\n";
print "<form name=\"assign_account_form\" method=post action=\"$site_dir/cgi-bin/assign_accounts2.cgi\">\n";
if ($mode == 1)
{
  print "<input type=hidden name=\"select_acct\" value=\"select_ledger\">\n";
  print "<input type=hidden name=\"acct_num\" value=\"$acct_num\">\n";
}
elsif ($mode == 2)
{
  print "<input type=hidden name=\"select_ledger\" value=\"select_ledger\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  if ($really_in_ledger eq "yes")
  {
    print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";
  }
}

print "<a href=\"\" onClick=\"SubmitForm(); return false;\">To assign accounts again...</a><br>\n";
print "</form>\n";

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

if ($really_in_ledger ne "yes")
{
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/assign_accounts.cgi\">Ledger Account Assignment</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\">Program Configuration</a></div>\n";
}
else
{
  print "<form name=\"form1\" method=post action=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";
  print "<input type=hidden name=\"select_ledger\" value=\"\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/assign_accounts2.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/assign_accounts2.cgi'); return false;\">Ledger Account Assignment</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/account_config.cgi'); return false;\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/prg_config.cgi'); return false;\">Program Configuration</a></div>\n";
  print "<input type=hidden name=\"open\" value=\"Open\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/ledger.cgi'); return false;\">Return to Ledger Summary</a></div>\n";
  print "</form>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;
