#!/usr/bin/perl
#$Id: close_fiscal_year.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# close_fiscal_year.cgi  -  Created by James Pattie, 12/31/1999
# This script begins the process of closing out the books for the current fiscal year.
# Based upon the type of company we are dealing with, we get the equity account(s) to
# work with first and then do all the computations and balance table entries in the
# next script(s).
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;  # The report module

use vars '$date';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

my $ledger_name;
my $ledger_db;
my $ledger_color;
my $business_type;
my $business_name;
my $report_obj;
my $ledger_code;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  A Ledger must be specified!</h1>\n";
  print "$sys_admin";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now we check and validate the ledger is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
$ledger_name = $result->getvalue(0, 0);

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# figure out the beginning and ending date of the current working year
my ($working_year, $from_date, $to_date, $q1_start, $q1_end, $q2_start, $q2_end, $q3_start, $q3_end, $q4_start, $q4_end);

$result = $conn->exec("select year, start_date, end_date, q1_start, q1_end, q2_start, q2_end, q3_start, q3_end, q4_start, q4_end from known_years_tb where year in (select working_year from variables_tb WHERE ledger_code = '$ledger_code') AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get working year and fiscal year dates from ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Program not configured!</h1>\n";
  print "Working Year is not set!<br>$sys_admin";
  CleanUp();
}
$working_year = $result->getvalue(0, 0);
$from_date    = $result->getvalue(0, 1);
$to_date      = $result->getvalue(0, 2);
$q1_start     = $result->getvalue(0, 3);
$q1_end       = $result->getvalue(0, 4);
$q2_start     = $result->getvalue(0, 5);
$q2_end       = $result->getvalue(0, 6);
$q3_start     = $result->getvalue(0, 7);
$q3_end       = $result->getvalue(0, 8);
$q4_start     = $result->getvalue(0, 9);
$q4_end       = $result->getvalue(0, 10);

$result = $conn->exec("SELECT business_type FROM ledger_tb WHERE code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the business type of this ledger.<br>$sys_admin";
  CleanUp();
}
$business_type = $result->getvalue(0, 0);

# get the business name
$result = $conn->exec("SELECT name FROM business_type_tb WHERE num = $business_type");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the business name for business type = '$business_type'.<br>$sys_admin";
  CleanUp();
}
$business_name = $result->getvalue(0, 0);

# get the last close out period value.
my $last_close_out;
$result = $conn->exec("SELECT DISTINCT(bal_date) FROM acct_bal_tb WHERE ledger_code = '$ledger_code' ORDER BY bal_date DESC");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the last close out date.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  $last_close_out = "Never";
}
else
{
  $last_close_out = $result->getvalue(0, 0);
}

my @AccountTypes;
@AccountTypes = ( 7 );
my @EquityAccounts;
@EquityAccounts = $report_obj->GetLocalAccounts(\@AccountTypes);
if ($report_obj->didErrorOccur)
{
  PrintErrorHeader();
  print "<h1>Error:  Getting Equity Accounts failed!</h1>\n";
  print $report_obj->getErrorMessage;
  CleanUp();
}

if ($business_type == 1 || $business_type == 3)
{
  if (scalar @{$EquityAccounts[0]} <= 1)
  {
    PrintErrorHeader();
    print <<"END_OF_STYLE";
    <style type="text/css"><!--
      a:hover   { color: red }
      a:link    { text-decoration: none; color: blue }
      a:visited { text-decoration: none; color: blue }
      a:active  { text-decoration: none; color: blue }-->
    </style>
END_OF_STYLE
    print "<h1>Error:  You need to assign at least 2 Equity Accounts to Ledger <font color=\"$ledger_color\">$ledger_name</font>!</h1>\n";
    print "<form method=post action=\"$site_dir/cgi-bin/assign_accounts2.cgi\" name=\"assign_accounts_form\">\n";
    print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
    print "<input type=hidden name=\"select_ledger\" value=\"\">\n";
    print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";
    print "<a href=\"$site_dir/cgi-bin/assign_accounts2.cgi\" onClick=\"javascript: document.assign_accounts_form.submit(); return false;\">Go Here to Assign Accounts</a>\n";
    print "</form>\n";
    CleanUp();
  }
}
else
{
  if (scalar @{$EquityAccounts[0]} == 0)
  {
    PrintErrorHeader();
    print <<"END_OF_STYLE";
    <style type="text/css"><!--
      a:hover   { color: red }
      a:link    { text-decoration: none; color: blue }
      a:visited { text-decoration: none; color: blue }
      a:active  { text-decoration: none; color: blue }-->
    </style>
END_OF_STYLE
    print "<h1>Error:  You need to assign Equity Account(s) to Ledger <font color=\"$ledger_color\">$ledger_name</font>!</h1>\n";
    print "<form method=post action=\"$site_dir/cgi-bin/assign_accounts2.cgi\" name=\"assign_accounts_form\">\n";
    print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
    print "<input type=hidden name=\"select_ledger\" value=\"\">\n";
    print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";
    print "<a href=\"$site_dir/cgi-bin/assign_accounts2.cgi\" onClick=\"javascript: document.assign_accounts_form.submit(); return false;\">Go Here to Assign Accounts</a>\n";
    print "</form>\n";
    CleanUp();
  }
}
# present them with the Select Equity Account screen

print "<head><title>$prog_name $prog_version Close Out Fiscal Year/Period in $working_year</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>

<script language="Javascript1.2">
  function EnterDate()
  {
    var d = document.close_fiscal_year_form;
    if (d.dates.selectedIndex == -1)
    {
      alert("You must select a Date for me to work with!");
      return;
    }
    d.close_date.value = d.dates.options[d.dates.selectedIndex].value;
  }

  function validate_form()
  {
    var d = document.close_fiscal_year_form;

    if (d.close_date.value.length > 0)
    {
      return true;
    }
    else
    {
      alert("You must specify a Date to Close Out to!");
      d.close_date.focus();
      return false;
    }
  }
</script>

END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

my ($i, $name, $code);

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><font color=red>Working in ledger <font color=\"$ledger_color\">$ledger_name</font>.  Closing out Fiscal Year/Period in <font color=\"black\">$working_year</font>.</font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#closing_fiscal_year\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);
print "<form method=\"post\" action=\"\" name=\"close_fiscal_year_form\">\n";
print "<input type=hidden name=\"ledger\" value=\"$input{'ledger'}\">\n";

print "<table border=0 cellpadding=2 cellspacing=0 width=50% bgcolor=\"$menu_bgcolor\">\n";
print "<tr><td align=center>Business Type = <b>$business_name</b>.</td></tr>\n";
print "<tr><td align=center>Last Close Out = <b>$last_close_out</b>.</td></tr>\n";
print "<tr><td align=center><hr></td></tr>\n";

my $IncomeSummaryNote;
$IncomeSummaryNote = "<tr><td align=center><i>Note:  Income summary is <b>Revenues</b> minus <b>Expenses</b> minus <b>Dividends Declared/Paid</b>.</i></td></tr>\n";
my $web_page;
$web_page = "close_fiscal_year2.cgi";

if ($business_type == 0 || $business_type == 2 || $business_type == 4)
{
  print "<tr><td align=center><font color=\"$menu_fgcolor\">Which equity account should be used for the income summary calculations?</font></td></tr>\n";
  print $IncomeSummaryNote;
  print "<tr><td align=center><select name=\"equity_account\">\n";
}
elsif ($business_type == 1)
{
  print "<tr><td align=center><font color=\"$menu_fgcolor\">Which equity accounts (partner equity) should be used for the income summary calculations?</font></td></tr>\n";
  print $IncomeSummaryNote;
  print "<tr><td align=center><i>Retained Earning results will be divided equally among the partners (accounts).</i></td></tr>\n";
  print "<tr><td align=center><select name=\"equity_account\" multiple size=5>\n";
}
elsif ($business_type == 3)
{
  $web_page = "divide_fiscal_year_proceeds.cgi";
  print "<tr><td align=center><font color=\"$menu_fgcolor\">Which equity accounts (shareholder equity) should be used for the income summary calculations?</font></td></tr>\n";
  print $IncomeSummaryNote;
  print "<tr><td align=center><i>Retained Earning results will be divided among the shareholders (accounts).</i></td></tr>\n";
  print "<tr><td align=center><select name=\"equity_account\" multiple size=5>\n";
}
else
{
  print "</table></form>\n";
  print "<h1>Error:  Unknown Business Type in ledger <font color=\"$ledger_color\">$ledger_name</font>!</h1>\n";
  print "Business Type specified = '$business_type'<br>$sys_admin";
  CleanUp();
}

# add the values of the equity accounts to the list and then close it up
my $i;
for ($i=0; $i < scalar @{$EquityAccounts[0]}; $i++)
{
  print "<option value=\"$EquityAccounts[0][$i][0]|$EquityAccounts[0][$i][2]\">$EquityAccounts[0][$i][2]\n";
}
print "</select></td></tr>\n";

if ($business_type == 1)
{
  print "<tr><td align=center><font color=\"$menu_fgcolor\">Which equity account (partner equity) should get the leftover cents?</font></td></tr>\n";
  print "<tr><td align=center><select name=\"leftover_equity_account\">\n";
}
elsif ($business_type == 3)
{
  print "<tr><td align=center><font color=\"$menu_fgcolor\">Which equity account (shareholder equity) should get the leftover cents?</font></td></tr>\n";
  print "<tr><td align=center><select name=\"leftover_equity_account\">\n";
}

if ($business_type == 1 || $business_type == 3)
{
  # add the values of the equity accounts to the list and then close it up
  my $i;
  for ($i=0; $i < scalar @{$EquityAccounts[0]}; $i++)
  {
    print "<option value=\"$EquityAccounts[0][$i][0]|$EquityAccounts[0][$i][2]\">$EquityAccounts[0][$i][2]\n";
  }
  print "</select></td></tr>\n";
}

# now display the enter date to close out at and a select box of possible values for the current fiscal year.
print "<tr><td align=center><font color=\"$menu_fgcolor\">Enter Date to CloseOut to:</font></td></tr>\n";
print "<tr><td align=center><input type=text name=\"close_date\" value=\"\"></td></tr>\n";
print "<tr><td align=center><select name=\"dates\" onChange=\"EnterDate();\">\n";
print "  <option value=\"$working_year\">$working_year\n";
print "  <option value=\"$q1_end\">$q1_end\n";
print "  <option value=\"$q2_end\">$q2_end\n";
print "  <option value=\"$q3_end\">$q3_end\n";
print "  <option value=\"$q4_end\">$q4_end\n";
print "  </select></td></tr>\n";

print "</table>\n";

print "<input type=submit name=\"next\" value=\"Next Step\" onClick=\"if (!validate_form()) return false; submit_my_form(document.close_fiscal_year_form, '$site_dir/cgi-bin/$web_page')\">&nbsp;<input type=reset value=\"Reset\">\n";
print "</form>\n";

if ($web_page eq "close_fiscal_year2.cgi")
{
  print "<br><br><font color=\"red\">Closing out the Books may take some time...</font><br>\n";
}

print "</center>\n";

print "</body>\n";
print "</html>\n";
exit 0;
