#$Id: cookie.pm,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# This is the cookie module for all our programs
package cookie;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw( %Cookies GetCookies SetCookieExpirationDate SetCookiePath SetCookieDomain SetCookies SetCompressedCookies GetCompressedCookies );

use strict;

my ( $Expiration_Date, $Path, $Domain, $Secure, @Encode_Chars, %Encode_Chars, @Decode_Chars, %Decode_Chars );
use vars '%Cookies';

# %Cookies is the hash of cookies found in the Server Environment.
# Since this will be exported it will be available to the calling program.

$Expiration_Date = "";  # if not defined, the cookie expires when the browser is closed.
    # if defined, the format is: WeekDay, dd-Mon-yyyy hh:mm:ss GMT

$Path = "";  # This is the path of the document as described by the cookie header.

$Domain = "";  # This is the host name of the server.

$Secure = "0"; # if set to 1, can only go across HTTPS connections.

# The list of Encode <=> Decode Characters to URL encode the cookie and it's data.

@Encode_Chars = ( '\%', '\+', '\;', '\,', '\=', '\&', '\:\:', '\s' );

%Encode_Chars = ( '\%' => '%25',
                  '\+' => '%2B',
                  '\;' => '%3B',
                  '\,' => '%2C',
                  '\=' => '%3D',
                  '\&' => '%26',
                  '\:\:' => '%3A%3A',
                  '\s' => '+'
                );

@Decode_Chars = ( '\+', '\%3A\%3A', '\%26', '\%3D', '\%2C', '\%3B', '\%2B', '\%25' );

%Decode_Chars = ( '\+' => ' ',
                  '\%3A\%3A' => '::',
                  '\%26' => '&',
                  '\%3D' => '=',
                  '\%2C' => ',',
                  '\%3B' => ';',
                  '\%2B' => '+',
                  '\%25' => '%'
                );

# GetCookies() - Gets Cookie(s) from the Server Environment.
# Takes: 	Array of Cookie Names or A Single Cookie Name or Nothing.
# Returns:	1 (If Successful), 0 (Otherwise)

sub GetCookies
{
  my @BrowserCookies = @_;
  my $cookie_found = 0;
  my ($cookie, $value);

  # check and make sure that the Browser sent us some cookies via the server.
  if ($ENV{'HTTP_COOKIE'})
  {
	# see if they requested a particular cookie(s).
    if (scalar @BrowserCookies > 0)
    {
      foreach my $temp (split(/; /,$ENV{'HTTP_COOKIE'}))
      {
        # Split the name=value pairs.

        ($cookie,$value) = split /=/, $temp;
        #print "cookie='$cookie', value='$value'\n\n";

        # do URL decoding
        foreach my $char (@Decode_Chars)
        {
          #print "char='$char', t='$Decode_Chars{$char}'\n";
          $cookie =~ s/$char/$Decode_Chars{$char}/g;
          $value =~ s/$char/$Decode_Chars{$char}/g;
        }

        # Check and see if this cookie was requested.
        foreach my $temp2 (@BrowserCookies)
        {
          if ($temp2 eq $cookie)
          {
            $Cookies{$cookie} = $value;
            $cookie_found = 1; # signal we found one of the specified cookies
          }
        }
      }
    }
    else # get all cookies passed in by the browser.
    {
      foreach my $temp (split(/; /,$ENV{'HTTP_COOKIE'}))
      {
        ($cookie,$value) = split /=/, $temp;

        # do URL decoding
        foreach my $char (@Decode_Chars)
        {
          $cookie =~ s/$char/$Decode_Chars{$char}/g;
          $value =~ s/$char/$Decode_Chars{$char}/g;
        }

        $Cookies{$cookie} = $value;
      }
      $cookie_found = 1; # signal we succeeded.
    }
  }

  return $cookie_found;
}

# SetCookieExpirationDate - Sets the expiration date for the cookie.
# Takes -	date in format (WeekDay, dd-Mon-yyyy hh:mm:ss GMT)
#                       Ex. "Fri, 04-Feb-2000 00:00:00 GMT"
# Returns -       1 - If successful and date passes check or 0 - If new Expiration Date was not set.

sub SetCookieExpirationDate
{
  my $date = $_[0];
  if ($date =~ /^\w{3}\,\s\d{2}\-\w{3}-\d{4}\s\d{2}\:\d{2}\:\d{2}\sGMT$/ ||
      $date eq "")
  {
    $Expiration_Date = $date;
    return 1;
  }
  else
  {
    return 0;
  }
}

# SetCookiePath - Set the path for the cookie.
# Takes - 	path - The path to set.  Ex. "/" or "/path/to/file"
# Returns -     Nothing.

sub SetCookiePath
{
  my $path = $_[0];
  $Path = $path;
}

# SetCookieDomain() - Sets the domain for the cookie to be sent to.  You can only specify a domain within the current domain.  Must have 2 or periods, depending on type of domain. e.g., .domain.com
# Takes -	domain - The domain to set the cookie to.   Ex. ".host.com"
# Returns -     1 - If successful, 0 - If unsuccessful.

sub SetCookieDomain
{
  my $domain = $_[0];
  if ($domain =~ /(.com|.edu|.net|.org|.gov|.mil|.int)$/i &&
      $domain =~ /\..+\.\w{3}$/)
  {
    $Domain = $domain;
    return 1;
  }
  elsif ($domain !~ /(.com|.edu|.net|.org|.gov|.mil|.int)$/i &&
         $domain =~ /\..+\..+\..+/)
  {
    $Domain = $domain;
    return 1;
  }
  else
  {
    return 0;
  }
}

# SetSecureCookie - This subroutine will set the cookie to be secure or in-secure.
# Takes -	flag - 0 or 1. 0 = insecure, 1 = secure
# Returns:       1 - If successful, 0 - If unsuccessful

sub SetSecureCookie
{
  my $flag = $_[0];

  if ($flag == 0 || $flag == 1)
  {
    $Secure = $flag;
    return 1;
  }
  else
  {
    return 0;
  }
}

# SetCookies - Sets one or more cookies by printing out the Set-Cookie HTTP header
# Takes -	name  - Name of the cookie to be set.
#               value - Value of the cookie to be set.
# Returns -     The string to print to set the cookie(s).

sub SetCookies
{
  my @cookies = @_;
  my ($cookie, $value, $char);
  my $result = "";

  while( ($cookie, $value) = @cookies )
  {
    # do URL encoding.

    foreach my $char (@Encode_Chars)
    {
      $cookie =~ s/$char/$Encode_Chars{$char}/g;
      $value =~ s/$char/$Encode_Chars{$char}/g;
    }

    # Print the Set-Cookie header with the cookie name and value, followed by semi-colon.

    $result .= "Set-Cookie: $cookie=$value;";

    # See if they want an Expiration Date set.

    if (length $Expiration_Date > 0)
    {
      $result .= " expires=$Expiration_Date;";
    }

    # See if they want a path set

    if (length $Path > 0)
    {
      $result .= " path=$Path;";
    }

    # See if they want a domain set

    if (length $Domain > 0)
    {
      $result .= " domain=$Domain;";
    }

    # See if they want it to be a secure cookie.

    if ($Secure)
    {
      $result .= " secure";
    }

    # Set the cookie by printing a \n (newline).

    $result .= "\n";

    shift(@cookies); # remove the items we just worked with.
    shift(@cookies);
  }
  return $result;
}

# SetCompressedCookies - This combines multiple cookies into one.
# Takes -	cookiename - Name of the cookie to be set.
#               name  - Name of the name=value pair.
#               value - Value of the name=value pair.
# Returns:      Nothing.

sub SetCompressedCookies
{
  my ($cookie_name, @cookies) = @_;
  my ($cookie, $value, $cookie_value);

  while ( ($cookie, $value) = @cookies )
  {
    #do URL encoding

    foreach my $char (@Encode_Chars)
    {
      $cookie =~ s/$char/$Encode_Chars{$char}/g;
      $value =~ s/$char/$Encode_Chars{$char}/g;
    }

    # Prepare the cookie value.  If a current cookie value exists, use
    # an ampersand (&) to separate the cookies and instead of using = to
    # separate the name and the value, use double colons (::), so it
    # won't confuse the browser.

    if ($cookie_value)
    {
      $cookie_value .= "&" . $cookie . "::" . $value;
    }
    else
    {
      $cookie_value = $cookie . "::" . $value;
    }

    shift(@cookies); # remove the items we just worked with.
    shift(@cookies);
  }

  # Use the SetCookies function to set the Multiple cookie. :)

  return SetCookies("$cookie_name","$cookie_value");
}

# GetCompressedCookies - This takes the compressed cookie names, and optionally the names of specific cookies you want returned
#                and uncompresses them, setting the values into %Cookies.  Specific names of cookies are optional and if not specified
#                all cookies found in the compressed cookie will be set.
# Takes -	cname - Name of the compressed cookie to be uncompressed.
#               names - Optional names of cookies to be returned from the
#                       compressed cookie if you don't want them all.
# Returns:       1 - If successful, 0 - If no cookies are retrieved.

sub GetCompressedCookies
{
  my ($cookie_name, @Cookies) = @_;
  my $cookie_found = 0;
  my ($ResultingCookie, $cookie, $value);

  if (GetCookies($cookie_name))
  { # we found the cookie in question
    if (scalar @Cookies > 0)
    {
      foreach my $temp (split(/&/, $Cookies{$cookie_name}))
      {
        # Split the cookie name and value pair.
        #print "\$temp = '$temp'\n";

        ($cookie, $value) = split /::/, $temp;

        # if we get a match, set the cookie.

        foreach $ResultingCookie (@Cookies)
        {
          if ($ResultingCookie eq $cookie)
          {
            $Cookies{$cookie} = $value;
            #print "\$Cookies{\$cookie} = '$Cookies{$cookie}', cookie = '$cookie', value = '$value'\n";
            $cookie_found = 1;
          }
        }
      }
    }
    else # get all cookies
    {
      foreach my $temp (split(/&/, $Cookies{$cookie_name}))
      {
        #print "\$temp = '$temp'\n";
        ($cookie, $value) = split /::/, $temp;

        $Cookies{$cookie} = $value;
        #print "\$Cookies{\$cookie} = '$Cookies{$cookie}', cookie = '$cookie', value = '$value'\n";
      }
      $cookie_found = 1;
    }

    #delete($Cookies{$cookie_name}); # remove the Compressed cookie so it is not used by accident.
  }

  return $cookie_found;
}

1;