#!/usr/bin/perl

# initial_balance_update.cgi  -  Created by Jason A. Pattie, 11/26/1999
# This script updates the beginning_balance_entry_tb table with the entries the user input
# on the previous form.  It also calculates the beginning balances for each account and
# stores them in the acct_balances_tb table.
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$count';
use vars '$count2';
use vars '$min_working_year';
use vars '$current_working_year';
use vars '$start_date';
use vars '$balance_date';
use vars '$fiscal_year';
use vars '$local_accounts_string';
use vars '$local_account_num';
use vars '@local_accounts_array';
use vars '%local_accounts_names';
use vars '%sub_accounts_names';
use vars '$debit_amount';
use vars '$credit_amount';
use vars '%account_balances';
use vars '$temp_account_type';
use vars '$temp_amount';

use vars '$record';
use vars '$item';
use vars '$name';
use vars '$value';
use vars '%initial_balances_values';
use vars '$balances_count';

use vars '$ref_num_element';
use vars '$account_element';
use vars '$sub_account_element';
use vars '$debit_element';
use vars '$credit_element';
use vars '$amount_element';
use vars '$dorc_element';

my $error_string;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

my $ledger_name;  # name of ledger to work with
my $ledger_db;
my $ledger_color;
my $result2;
my $ledger_code;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}
$error_string = IsValidString($ledger_db);
if (length $error_string > 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Database Name!</h1>\n";
  print "<b>$error_string</b> is invalid!<br>$back_button";
  CleanUp();
}

if (not exists $input{'fiscal_year'})
{
  PrintErrorHeader();
  print "<h1>Error: Access denied!</h1>\n";
  print "Missing necessary form value (fiscal year).<br>$sys_admin";
  CleanUp();
}

if (not exists $input{'equity_account_select'})
{
  PrintErrorHeader();
  print "<h1>Error: Access denied!</h1>\n";
  print "Missing necessary form value (equity account).<br>$sys_admin";
  CleanUp();
}

if ($input{'equity_account_select'} eq "-1" || $input{'equity_account_select'} eq "")
{
  PrintErrorHeader();
  print "<h1>Error: Access denied!</h1>\n";
  print "An equity account has not been chosen!<br>$back_button";
  CleanUp();
}

if (not exists $input{'initial_balances_values'})
{
  PrintErrorHeader();
  print "<h1>Error: Access denied!</h1>\n";
  print "A necessary form value (initial balances) is missing.<br>$sys_admin";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now validate that the ledger they specified is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# check to see if the current fiscal year is the first year the accounting package is being used
$result = $conn->exec("select min(year) from known_years_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>1008: Tried to get the minimum known year from the known_years_tb.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0 || $result->ntuples > 1)
{
  PrintErrorHeader();
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "1009: " . $result->ntuples . " records were returned.  There should be only one.<br>$sys_admin";
  CleanUp();
}
$min_working_year = $result->getvalue(0, 0);
$result = $conn->exec("select working_year from variables_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>1010: Tried to get the current working year from the variables_tb table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0 || $result->ntuples > 1)
{
  PrintErrorHeader();
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "1011: " . $result->ntuples . " records were returned.  There should be only one.<br>$sys_admin";
  CleanUp();
}
$current_working_year = $result->getvalue(0, 0);
if ($min_working_year != $current_working_year)
{
  PrintErrorHeader();
  print "<h1>Error: Access denied!</h1>\n";
  print "1016: The initial balances for accounts cannot be created or edited after the first year the system has been in use.<br>$sys_admin";
  CleanUp();
}
# get the date to use to enter account balances into the journal entry and debit/credit tables
$fiscal_year = $input{'fiscal_year'};
if ($fiscal_year != $current_working_year)
{
  PrintErrorHeader();
  print "<h1>Error: Access denied!</h1>\n";
  print "1022: The fiscal year passed through is not equal to the current working year.<br>$sys_admin";
  CleanUp();
}
$result = $conn->exec("select start_date from known_years_tb where year = $fiscal_year AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>1023: Tried to get the beginning date for the fiscal year $fiscal_year.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: No records returned!</h1>\n";
  print "1024: Attempted to retrieve the start date for the fiscal year $fiscal_year.<br>$sys_admin";
  CleanUp();
}
$start_date = $result->getvalue(0, 0);
$start_date =~ s/-/\//g;
$balance_date = `/bin/date -d "$start_date 1 day ago" +%m-%d-%Y`;
chomp $balance_date;

# get the data from the previous script to update the journal entry and debit/credit tables with
$count = 0;
foreach $record (split /\;/, $input{'initial_balances_values'})
{
  foreach $item (split /\|/, $record)
  {
    ($name, $value) = split /\=/, $item, 2;
    $initial_balances_values{"$count - $name"} = $value;
    if ($name eq "ac")
    {
      $local_accounts_array[$count] .= $value;
    }
    if ($name eq "sa" && $value ne "-1")
    {
      $sub_accounts_names{$value} = "";
    }
  }
  $count ++;
}
$balances_count = $count;

$local_accounts_string = "";
for ($count=0; $count < @local_accounts_array; $count++)
{
  $local_accounts_string .= $local_accounts_array[$count];
  if ($count < @local_accounts_array - 1)
  {
    $local_accounts_string .= ", ";
  }
}
# add the equity account number to the list of accounts to retrieve names for
$local_accounts_string .= ", " .$input{'equity_account_select'};
# lookup the names for the local list of accounts from the main database
$result2 = $conn->exec("select name, number from account_tb where number in ($local_accounts_string) order by name");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>1019: Tried to get the names for the local list of accounts.<br>$sys_admin";
  CleanUp();
}
if ($result2->ntuples > 0)
{
  for ($count=0; $count < $result2->ntuples; $count++)
  {
    # load hash with local account names
    $local_account_num = $result2->getvalue($count, 1);
    $local_accounts_names{$local_account_num} = $result2->getvalue($count, 0) . " - " . $local_account_num;
    # initialize the account_balances hash entry "local_account_num - -2" to zero
    $account_balances{"$local_account_num - -2"} = 0.0;
  }
}
else
{
  PrintErrorHeader();
  print "<h1>Error: No account names returned!</h1>\n";
  print "1020: Tried to get the names for the local accounts from the main database.  None were returned.<br>($local_accounts_string)<br>Perhaps the main database does not have any accounts?<br>$sys_admin";
  CleanUp();
}
# lookup the names of the sub-accounts that were used
foreach $count (keys %sub_accounts_names)
{
  $result = $conn->exec("select sub_acc_name from sub_acc_tb where sub_acc_number = $count");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing database failed!</h1>\n";
    print "Tried to lookup the name for sub-account: $count.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples <= 0)
  {
    PrintErrorHeader();
    print "<h1>Error: No records returned!</h1>\n";
    print "Tried to lookup the name for sub-account: $count.<br>$sys_admin";
    CleanUp();
  }
  $sub_accounts_names{$count} = $result->getvalue(0, 0);
}
# assign a value of "<i>Other</i>" to the name for sub-account -1
$sub_accounts_names{'-1'} = "<i>Other</i>";

# use a transaction to do this work
$result = $conn->exec("begin work");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Could not start transaction!</h1>\n";
  print $conn->errorMessage . "<br>1025: Attempted to start a transaction to do the deleting from and updating to the beginning_balance_entry_tb and acct_bal_tb tables.<br>$sys_admin";
  CleanUp();
}
$transaction = 1;
# delete all entries from the beginning_balance_entry_tb table
$result = $conn->exec("delete from beginning_balance_entry_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  if ($transaction == 1)
  {
    $result = $conn->exec("rollback work");
  }
  PrintErrorHeader();
  print "<h1>Error: Deleting entries failed!</h1>\n";
  print $conn->errorMessage . "<br>1026: Attempted to delete all entries from the beginning_balance_entry_tb table.<br>$sys_admin";
  CleanUp();
}

# run through each row to determine which amounts to use to calculate the beginning balance for each local account
# add each row to the beginning_balance_entry_tb table utilizing the ref_num associated with the row
for ($count=0; $count < $balances_count; $count++)
{
  $name = "$count - rf";
  $ref_num_element = $initial_balances_values{$name};

  $name = "$count - ac";
  $account_element = $initial_balances_values{$name};

  $name = "$count - sa";
  $sub_account_element = $initial_balances_values{$name};

  $name = "$count - db";
  $debit_element = $initial_balances_values{$name};

  $name = "$count - cr";
  $credit_element = $initial_balances_values{$name};

  if ($debit_element ne "")
  {
    $amount_element = $debit_element;
    $dorc_element = "true";
    # sum the "debits" for each local account from the form elements
    $debit_amount = $amount_element;
    $credit_amount = 0.0;
  }
  elsif ($credit_element ne "")
  {
    $amount_element = $credit_element;
    $dorc_element = "false";
    # sum the "credits" for each local account from the form elements
    $credit_amount = $amount_element;
    $debit_amount = 0.0;
  }
  else
  {
    if ($transaction == 1)
    {
      $result = $conn->exec("rollback work");
    }
    PrintErrorHeader();
    print "<h1>Error: Debit and Credit value missing!</h1>\n";
    print "A debit or credit value was not parsed for account: $account_element, sub_account: $sub_account_element.<br>$sys_admin";
    CleanUp();
  }

  # insert the row into beginning_balance_entry_tb table
  $result = $conn->exec("insert into beginning_balance_entry_tb (ref_num, account_num, sub_account_num, amount, dorc, ledger_code) values ($ref_num_element, $account_element, $sub_account_element, $amount_element, $dorc_element, '$ledger_code')");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    if ($transaction == 1)
    {
      $result = $conn->exec("rollback work");
    }
    PrintErrorHeader();
    print "<h1>Error: Inserting entries failed!</h1>\n";
    print $conn->errorMessage . "<br>1029: Tried to insert the record for ref_num: $ref_num_element, account: $account_element, sub-account: $sub_account_element, amount: $amount_element, dorc: $dorc_element, fiscal_year:" . $input{"fiscal_year"} . "<br>$sys_admin";
    CleanUp();
  }

  # for each account, determine whether to subtract the debits from the credits or the credits from the debits by using the
  # debit_increases flag

  # get the type of the account
  $result = $conn->exec("select type from account_tb where number = $account_element");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    if ($transaction == 1)
    {
      $result = $conn->exec("rollback work");
    }
    PrintErrorHeader();
    print "<h1>Error: Getting account type info. failed!</h1>\n";
    print $conn->errorMessage . "<br>1030: Tried to get the account type info. for account: $account_element.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples != 1)
  {
    if ($transaction == 1)
    {
      $result2 = $conn->exec("rollback work");
    }
    PrintErrorHeader();
    print "<h1>Error: Wrong number of records returned!</h1>\n";
    print "1031: " . $result->ntuples . " records were returned for account: $account_element.  There should be only one.<br>$sys_admin";
    CleanUp();
  }
  $temp_account_type = $result->getvalue(0, 0);

  # lookup in the database the debit_increases flag for the account and use it to subtract the debits from the credits
  # or vice versa
  $result = $conn->exec("select debit_increases from acct_type_tb where num = $temp_account_type");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    if ($transaction == 1)
    {
      $result = $conn->exec("rollback work");
    }
    PrintErrorHeader();
    print "<h1>Error: Getting 'debit_increases' flag failed!</h1>\n";
    print $conn->errorMessage . "<br>1032: Tried to get the debit_increases flag from the database for account type: $temp_account_type.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples != 1)
  {
    if ($transaction == 1)
    {
      $result2 = $conn->exec("rollback work");
    }
    PrintErrorHeader();
    print "<h1>Error: Wrong number of records returned!</h1>\n";
    print "1033: " . $result->ntuples . " records were returned.  There should be only one.<br>$sys_admin";
    CleanUp();
  }
  if ($result->getvalue(0, 0) eq "t")
  {
    $temp_amount = $debit_amount - $credit_amount;
    $account_balances{"$account_element - $sub_account_element"} += $temp_amount;
    $account_balances{"$account_element - -2"} += $temp_amount;
  }
  elsif ($result->getvalue(0, 0) eq "f")
  {
    $temp_amount = $credit_amount - $debit_amount;
    $account_balances{"$account_element - $sub_account_element"} += $temp_amount;
    $account_balances{"$account_element - -2"} += $temp_amount;
  }
}
# account for the equity account's values
$account_balances{$input{'equity_account_select'} . " - -2"} += $input{'equity_credit_amount'} - $input{'equity_debit_amount'};
$account_balances{$input{'equity_account_select'} . " - -1"} += $input{'equity_credit_amount'} - $input{'equity_debit_amount'};

# remove all account balances from the acct_bal_tb (journal entry; debit/credit ??) table that have a balance date of balance_date
$result = $conn->exec("delete from acct_bal_tb where bal_date = '$balance_date' AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  if ($transaction == 1)
  {
    $result = $conn->exec("rollback work");
  }
  PrintErrorHeader();
  print "<h1>Error: Deletion of records failed!</h1>\n";
  print $conn->errorMessage . "<br>1034: Tried to delete all records for date $balance_date from acct_bal_tb table.<br>$sys_admin";
  CleanUp();
}

# use the balance_date variable that should contain the date of the day prior to the start of the fiscal year to enter the
# account balances into the acct_bal_tb table
foreach $name (keys %account_balances)
{
  ($account_element, $sub_account_element) = split " - ", $name, 2;
  $result = $conn->exec("insert into acct_bal_tb (acct_num, sub_acct_num, bal_date, balance, ledger_code) values ($account_element, $sub_account_element, '$balance_date', $account_balances{$name}, '$ledger_code')");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    if ($transaction == 1)
    {
      $result = $conn->exec("rollback work");
    }
    PrintErrorHeader();
    print "<h1>Error: Failed to insert balance records!</h1>\n";
    print $conn->errorMessage . "<br>1035: Tried to insert an account - sub_account ($name) balance into the acct_bal_tb table: balance_date=$balance_date, balance=$account_balances{$name}.<br>$sys_admin";
    CleanUp();
  }
}

# commit the work
$result = $conn->exec("commit work");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  if ($transaction == 1)
  {
    $result = $conn->exec("rollback work");
  }
  PrintErrorHeader();
  print "<h1>Error: Committing transaction failed!</h1>\n";
  print $conn->errorMessage . "<br>1027: Tried to commit the transaction, but something went wrong.<br>$sys_admin";
  CleanUp();
}
$transaction = 0;

print "<head><title>$prog_name $prog_version Initial Account Balance Activity</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=left width=47%><font color=\"red\">";
print "Working with Initial Account Balances in Ledger <font color=\"$ledger_color\">$ledger_name</font></font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#initial_balance\" target=\"docs\"><img src=\"$site_dir/images/help.jpg\" alt=\"Help\" border=0></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);
# display balance date
print "<center><h2>Initial Account Balances for <font color=\"blue\">$balance_date</font></h2></center>\n";
# display the final balances for each account
print "<table width=50% border=1>\n";
print "<tr>\n";
print "<th>Account/Sub-account</th><th>Balance</th>\n";
print "</tr>\n";
sub namefromnum {
  my $account_a;
  my $sub_account_a;
  my $account_b;
  my $sub_account_b;

  ($account_a, $sub_account_a) = split " - ", $a, 2;
  ($account_b, $sub_account_b) = split " - ", $b, 2;
  "$local_accounts_names{$account_a}/$sub_accounts_names{$sub_account_a}" cmp "$local_accounts_names{$account_b}/$sub_accounts_names{$sub_account_b}";
}
foreach $name (sort namefromnum keys %account_balances)
{
  ($account_element, $sub_account_element) = split " - ", $name, 2;
  print "<tr>\n";
  print "<td><font size=-1>" . $local_accounts_names{$account_element} . "/" . $sub_accounts_names{$sub_account_element} . " - ";
  if ($sub_account_element < 0)
  {
    print "(";
  }
  print $sub_account_element;
  if ($sub_account_element < 0)
  {
    print ")";
  }
  print "</font></td>\n";
  print "<td width=25% align=right><font size=-1>";
  if ($account_balances{$name} < 0.00)
  {
    print "<font color=\"red\">(" . commify(sprintf("%.2f", abs($account_balances{$name}))) . ")</font>";
  }
  else
  {
    print commify(sprintf("%.2f", abs($account_balances{$name}))) . "&nbsp;&nbsp;";
  }
  print "</font></td>\n";
  print "</tr>\n";
}
print "</table>\n";

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";
print "</center>\n";

print "<form method=post action=\"$site_dir/cgi-bin/initial_balance_sheet.cgi\" name=\"init_balance_form\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<a href=\"$site_dir/cgi-bin/initial_balance.cgi\" onClick=\"javascript: document.init_balance_form.submit(); return false;\">Re-Edit Initial Account Balances</a><br>\n";
print "</form>\n";

print "</body>\n";
print "</html>\n";
exit 0;
