#!/usr/bin/perl
#$Id: journals.cgi,v 1.4 2003/04/09 20:04:39 pcxuser Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# journals.cgi  -  Created by James Pattie, 06/05/1999
# This script presents the user with the main menu for the journals.
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

my $ledger_name;
my $ledger_db;
my $ledger_color;
my $journal_name;
my $journal_code;
my $ledger_code;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
 $dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
 $database_connected = 0;

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  A Ledger must be specified!</h1>\n";
  print "$sys_admin";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now we check and validate the ledger is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
$ledger_name = $result->getvalue(0, 0);

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# figure out the beginning and ending date of the current working year
my ($working_year, $from_date, $to_date);

$result = $conn->exec("select year, start_date, end_date from known_years_tb where year in (select working_year from variables_tb WHERE ledger_code = '$ledger_code') AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get working year and fiscal year dates from ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Program not configured!</h1>\n";
  print "Working Year is not set!<br>$sys_admin";
  CleanUp();
}
$working_year = $result->getvalue(0, 0);
$from_date    = $result->getvalue(0, 1);
$to_date      = $result->getvalue(0, 2);

$result = $conn->exec("select name, code from journal_tb order by name");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get list of journals available.<br>$sys_admin";
  CleanUp();
}

# present them with the main menu

print "<head><title>$prog_name $prog_version Journals Main Menu</title>\n";
print "</head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><font color=red>Working in ledger <font color=\"$ledger_color\">$ledger_name</font>. &nbsp;FY is <font color=\"black\">$working_year</font>.</font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#journals_main_menu\" target=\"docs\"><img src=\"$site_dir/images/help.jpg\" alt=\"Help\" border=0</a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Journals Main Menu</h2>\n";

my ($i, $name, $code);
if ($result->ntuples > 0)
{
  print "<form method=\"post\" action=\"\" name=\"journals_form\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$input{'ledger'}\">\n";
  print "<input type=hidden name=\"journal_view_command\" value=\"\">\n";
  print "<table border=1 cellspacing=0 cellpadding=$table_cellpadding bgcolor=\"$menu_bgcolor\">\n";
  print "<tr><td align=left>\n";
  print "Journal&nbsp;";
  print "<select name=\"journal\">\n";
  print "<option value=\"all_journals\">All Journals\n";
  for ($i=0; $i < $result->ntuples; $i++)
  {
    $name = $result->getvalue($i, 0);
    $code = $result->getvalue($i, 1);
    print "<option value=\"$name|$code\">$name ($code)\n";
  }

  print "</select>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\n";
#took out delete button for journals JT 12/8/00
#  if ($result->ntuples > 1)
#  {
#    print "<font color=\"$menu_fgcolor\"><input type=button name=\"delete\" value=\"Delete\" onClick=\"document.journals_form.journal_view_command.value='delete';submit_my_form(document.journals_form, '$site_dir/cgi-bin/journal_mod1.cgi');document.journals_form.submit();\"></font>\n";
#  }
  print "<br><br>Date:&nbsp;&nbsp; From &nbsp;<font color=\"white\">$from_date</font><input type=hidden name=\"date_from\" value=\"$from_date\">&nbsp;&nbsp;To &nbsp;<font color=\"white\">$to_date</font><input type=hidden name=\"date_to\" value=\"$to_date\">";
#  print "&nbsp;&nbsp;&nbsp;&nbsp;Range ";
#  print "<select name=\"range\">\n";
#  print "</select>\n";
  print "<br>Sort By &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<select name=\"sort_by\">\n";
  print "<option value=\"date\">Date\n";
  print "<option value=\"ref_num\">Reference Number\n";
  print "<option value=\"payee_payer\">Payee Payer\n";
  print "</select> And &nbsp;&nbsp;<input type=checkbox name=\"sort_journal\" value=\"ON\"> Journals<br>\n";
  print "How many entries to display at the same time:&nbsp;<select name=\"num_entries_to_display\">\n";
  print "<option value=\"0\">all\n";
  print "<option value=\"5\">5\n";
  print "<option value=\"10\">10\n";
  print "<option value=\"15\">15\n";
  print "<option value=\"20\" selected>20\n";
  print "<option value=\"25\">25\n";
  print "<option value=\"30\">30\n";
  print "<option value=\"35\">35\n";
  print "<option value=\"40\">40\n";
  print "<option value=\"45\">45\n";
  print "<option value=\"50\">50\n";
  print "</select><input type=hidden name=\"start_index\" value=\"1\"><br>\n";
  print "<input type=\"hidden\" name=\"app\" value=\"Accounting\">\n";
  print "<input type=\"hidden\" name=\"state\" value=\"InvJournal\">\n";
  print "<input type=\"hidden\" name=\"command\" value=\"display\">\n";
  print "<input type=\"hidden\" name=\"ledgerCode\" value=\"$ledger_code\">\n";
  print "<font color=\"$menu_fgcolor\">";
  print "<input type=button name=\"open\" value=\"Open\" onClick=\"document.journals_form.journal_view_command.value='open';submit_my_form(document.journals_form, (document.journals_form.journal.options[document.journals_form.journal.selectedIndex].value == 'Investment|IS')?'/portal/cgi-bin/index.cgi?app=Accounting&state=InvJournal&command=display&ledger=$ledger_db':'$site_dir/cgi-bin/journal_mod1.cgi' );document.journals_form.submit();\">&nbsp;";
  print "<input type=reset value=\"Reset\">";
  print "</font>";
  print "</td></tr></table>\n";
  print "</form>\n";
}
print "<br><br><br>\n";
#JT  2001-06-27, took out ability to create journals
#print "<form method=\"post\" action=\"\" name=\"journals_form_create\" onSubmit=\"return ValidateCreateForm();\">\n";
#print "<input type=hidden name=\"ledger\" value=\"$input{'ledger'}\">\n";
#print "<input type=hidden name=\"journal_view_command\" value=\"\">\n";
#print "<h3>Create new Journal</h3>\n";
#print "<table border=1 cellspacing=0 cellpadding=$table_cellpadding bgcolor=\"$menu_bgcolor\">\n";
#print "<tr><td align=left>\n";
#print "Name &nbsp;<input type=text name=\"name\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Code <input type=text name=\"code\"><br>\n";
#print "<font color=\"$menu_fgcolor\"><input type=button name=\"add\" value=\"Add New\" onClick=\"if (!ValidateCreateForm()) return false;document.journals_form_create.journal_view_command.value='add';submit_my_form(document.journals_form_create, '$site_dir/cgi-bin/journal_mod1.cgi');document.journals_form_create.submit();\">&nbsp;<input type=reset value=\"Reset\"></font>\n";
#print "</td></tr></table>\n";
#print "</form>\n";

print "<table border=0 cellpadding=0 cellspacing=0>\n";
print "<tr>\n";
# Display the Search Button
print "<td><form method=\"post\" action=\"\" name=\"journals_form_search\">\n";
print "<input type=hidden name=\"ledger\" value=\"$input{'ledger'}\">\n";
print "<input type=hidden name=\"journal_view_command\" value=\"\">\n";
print "<font color=\"$menu_fgcolor\"><input type=button name=\"search\" value=\"Search Journal\" onClick=\"document.journals_form_search.journal_view_command.value='search';submit_my_form(document.journals_form_search, '$site_dir/cgi-bin/journal_search.cgi');document.journals_form_search.submit();\"></font>\n";
print "</form></td>\n";

# create the form
print "<td><form method=post action=\"$site_dir/cgi-bin/journal_activity.cgi\" name=\"journal_view\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<input type=hidden name=\"journal_command\" value=\"\">\n";
print "<input type=hidden name=\"journal_code\" value=\"all_journals\">\n";
print "<input type=hidden name=\"journal_bgcolor\" value=\"\">\n";
print "<font color=\"$menu_fgcolor\"><input type=button name=\"create\" value=\"New Transaction\" onClick=\"document.journal_view.journal_command.value='create';document.journal_view.submit();\"></font>";
print "</form></td>\n";

# Display the Print Button
print "<td><form method=post action=\"$site_dir/cgi-bin/journal_print.cgi\" name=\"journal_print\">\n";
print "<input type=hidden name=\"ledger\" value=\"$input{'ledger'}\">\n";
print "<font color=\"$menu_fgcolor\"><input type=button name=\"print\" value=\"Print Journal\" onClick=\"document.journal_print.submit();\"></font>";
print "</form></td>\n";
print "</tr></table>\n";

print "</center>\n";

print "</body>\n";
print "</html>\n";
exit 0;
