#!/usr/bin/perl
#$Id: prg_config.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# prg_config.cgi  -  Created by James Pattie, 05/18/1999
# This script allows the user to configure the Accounting package.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$report';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";
print "<head><title>$prog_name $prog_version Program Configuration</title></head>\n";
print "<body>\n";

my $ledger_db;
if (exists $input{'ledger'})
{
  $ledger_db = $input{'ledger'};
}
use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# see if there are any ledgers to work with.
$result = $conn->exec("SELECT * FROM ledger_tb");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Checking for Ledgers failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}
my $ledgers_exist;
$ledgers_exist = $result->ntuples;

# create the reports object
$report = reports->new($conn, $ledger_db);
if ($report->didErrorOccur())
{
  PrintErrorHeader();
  print $report->getErrorMessage();
  CleanUp();
}
my $ledger_code;
$ledger_code = $report->getLedgerCode();
if ($report->didErrorOccur())
{
  PrintErrorHeader();
  print $report->getErrorMessage();
  CleanUp();
}

# present them with the main menu 

print "<head><title>$prog_name $prog_version Program Configuration</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><br></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#program_configuration\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Program Configuration</h2>\n";

print "<br><br>\n";
if (length $ledger_db == 0)
{
  print "<a href=\"$site_dir/cgi-bin/account_config.cgi\">Account Configuration</a><br><br>\n";
  print "<a href=\"$site_dir/cgi-bin/sub_accounts_config2.cgi\">Sub Account Configuration</a><br><br>\n";
  if ($ledgers_exist)
  {
    print "<a href=\"$site_dir/cgi-bin/time_tracker_config.cgi\">Time Tracker Configuration</a><br>\n";
  }
}
else
{
  # see if there are any close out dates that are inside the current fiscal year
  $result = $conn->exec("SELECT DISTINCT(ab.bal_date) from acct_bal_tb as ab, known_years_tb as k, variables_tb as v WHERE ab.ledger_code = '$ledger_code' AND ab.ledger_code = k.ledger_code AND k.ledger_code = v.ledger_code AND k.year = v.working_year AND ab.bal_date >= k.start_date AND ab.bal_date < k.end_date");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "</td></tr></table>\n";
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to see if they have any closed out periods in the current fiscal year.<br>$sys_admin";
    CleanUp();
  }
  my $latest_closeout = "";
  if ($result->ntuples > 0)
  {
    $latest_closeout = $result->getvalue(0,0);
  }

  # if the current working year is the lowest year in the known years table, display the link to allow QIF imports
  $result = $conn->exec("select min(year) from known_years_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "</form></td></tr></table>\n";
    print "<h1>Error: Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>1008: Tried to get the minimum known year from the known_years_tb.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0 || $result->ntuples > 1)
  {
    print "</form></td></tr></table>\n";
    print "<h1>Error: Wrong number of records returned!</h1>\n";
    print "1009: " . $result->ntuples . " records were returned.  There should be only one.<br>$sys_admin";
    CleanUp();
  }
  my $min_working_year = $result->getvalue(0, 0);
  $result = $conn->exec("select working_year from variables_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "</form></td></tr></table>\n";
    print "<h1>Error: Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>1010: Tried to get the current working year from the variables_tb.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0 || $result->ntuples > 1)
  {
    print "</form></td></tr></table>\n";
    print "<h1>Error: Wrong number of records returned!</h1>\n";
    print "1011: " . $result->ntuples . " records were returned.  There should be only one. ledger_code = '$ledger_code'<br>$sys_admin";
    CleanUp();
  }
  my $current_working_year = $result->getvalue(0, 0);

  print "<form method=post action=\"\" name=\"prg_form\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<input type=hidden name=\"open_accounts\" value=\"\">\n";
  print "<input type=hidden name=\"entity\" value=\"-ledger\">\n";
  print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";
  print "<input type=hidden name=\"app\" value=\"Accounting\">\n";
  if ($min_working_year == $current_working_year && $latest_closeout eq "")
  {
    print "<input type=hidden name=\"state\" value=\"QIFImport\">\n";
  }
  print "<input type=hidden name=\"command\" value=\"display\">\n";
  print "<a href=\"$site_dir/cgi-bin/account_config.cgi\" onClick=\"go_here(document.prg_form, '$site_dir/cgi-bin/account_config.cgi'); return false;\">Account Configuration</a><br><br>\n";
  print "<a href=\"$site_dir/cgi-bin/sub_accounts_config2.cgi\" onClick=\"go_here(document.prg_form, '$site_dir/cgi-bin/sub_accounts_config2.cgi'); return false;\">Sub Account Configuration</a><br><br>\n";
  if ($min_working_year == $current_working_year && $latest_closeout eq "")
  {
    print "<a href=\"/portal/cgi-bin/index.cgi\" onClick=\"go_here(document.prg_form, '/portal/cgi-bin/index.cgi'); return false;\">Import .QIF Data File</a><br><br>\n";
  }
  if ($ledgers_exist)
  {
    print "<a href=\"$site_dir/cgi-bin/time_tracker_config.cgi\" onClick=\"go_here(document.prg_form, '$site_dir/cgi-bin/time_tracker_config1.cgi'); return false;\">Time Tracker Configuration</a><br>\n";
  }
  print "</form>\n";
}

if (length $ledger_db == 0)
{
}
else
{
  # print a horizontal rule to break the menu from the behind the scenes links
  print "<br><hr>\n";
  print "<form name=\"form1\" method=post action=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<input type=hidden name=\"open\" value=\"Open\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/ledger.cgi'); return false;\">Return to Ledger Summary</a></div>\n";
  print "</form>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;