#!/usr/bin/perl
#$Id: reopen_fiscal_year.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# reopen_fiscal_year.cgi  -  Created by James Pattie, 01/05/2000
# This script begins the process of re-opening the books for a previous fiscal year.
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;  # The report module

use vars '$date';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

my $ledger_name;
my $ledger_db;
my $ledger_color;
my $business_type;
my $report_obj;
my $ledger_code;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  A Ledger must be specified!</h1>\n";
  print "$sys_admin";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now we check and validate the ledger is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
$ledger_name = $result->getvalue(0, 0);

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# figure out the beginning and ending date of the current working year
my ($working_year, $from_date, $to_date);

$result = $conn->exec("select year, start_date, end_date from known_years_tb where year in (select working_year from variables_tb WHERE ledger_code = '$ledger_code') AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get working year and fiscal year dates from ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Program not configured!</h1>\n";
  print "Working Year is not set!<br>$sys_admin";
  CleanUp();
}
$working_year = $result->getvalue(0, 0);
$from_date    = $result->getvalue(0, 1);
$to_date      = $result->getvalue(0, 2);

# get a list of all valid fiscal years available.
$result = $conn->exec("SELECT year FROM known_years_tb where year < $working_year AND ledger_code = '$ledger_code' ORDER BY year DESC");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get list of available Fiscal Years.<br>$sys_admin";
  CleanUp();
}

# get a list of all closeout dates that are within the current fiscal period.
my $result2;
$result2 = $conn->exec("SELECT DISTINCT(bal_date) FROM acct_bal_tb WHERE ledger_code = '$ledger_code' AND ledger_code = known_years_tb.ledger_code AND known_years_tb.ledger_code = variables_tb.ledger_code AND known_years_tb.year = variables_tb.working_year AND bal_date >= known_years_tb.start_date AND bal_date < known_years_tb.end_date ORDER BY bal_date DESC");
if ($result2->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get list of all Close Out Periods in the current fiscal year.<br>$sys_admin";
  CleanUp();
}

if ($result->ntuples == 0 && $result2->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Internal Error!</h1>\n";
  print "There are no available Fiscal Years/Periods to choose from!<br>$sys_admin";
  CleanUp();
}

print "<head><title>$prog_name $prog_version Re-Open A Fiscal Year</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><font color=red>Working in ledger <font color=\"$ledger_color\">$ledger_name</font>.  Current Fiscal Year = <font color=\"black\">$working_year</font></font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#reopen_fiscal_year\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);
print "<form method=\"post\" action=\"$site_dir/cgi-bin/reopen_fiscal_year2.cgi\" name=\"open_fiscal_year_form\">\n";
print "<input type=hidden name=\"ledger\" value=\"$input{'ledger'}\">\n";

print "<table border=0 cellpadding=2 cellspacing=0 bgcolor=\"$menu_bgcolor\">\n";
print "<tr><th>Choose Fiscal Year/Period</th></tr>\n";
print "<tr><td align=center><select name=\"period\">\n";
my $i;
for (my $j=0; $j < $result2->ntuples; $j++)  # print any Fiscal Periods first as they are in the Current Fiscal Year.
{
  print "<option value=\"" . $result2->getvalue($j, 0) . "\">" . $result2->getvalue($j, 0) . "\n";
}
for ($i=0; $i < $result->ntuples; $i++)  # Then print any Fiscal Years as they should be after any Fiscal Periods to reopen.
{
  my $year = $result->getvalue($i, 0);
  print "<option value=\"$year\">$year\n";
}
print "</select></td></tr>\n";
print "</table>\n";

print "<input type=submit name=\"next\" value=\"Next Step\">&nbsp;<input type=reset value=\"Reset\">\n";
print "</form>\n";

print "</center>\n";

print "</body>\n";
print "</html>\n";
exit 0;
