#!/usr/bin/perl
# report.cgi - This program is the front to the reporting system.  It displays the main reporting menus.
#$Id: report.cgi,v 1.13 2002/10/21 21:12:38 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# Program Modes:  Uses command
# command
#	  = 'display_for_running' - List all created reports to allow the user to customize and then run.  Include memorized reports.
#	  = 'bottom' - Generates a blank page.  Used for the framesets so that the bottom frame starts out empty.

use strict;
use cgi_lib;
use custom;

my %errors;
%errors = ( 4000 => 'Eval of HTMLObject::FrameSet failed',
	       4001 => 'Invalid command specified',
	       4002 => 'Eval of HTMLObject::Normal failed',
	       4003 => 'command must be specified',
	       4004 => "Couldn't connect to Database",
	       4005 => "Parameter requires a value",
	       4006 => "Invalid Value for Parameter",
	       4007 => "DBI Preparation of SQL Statement Failed",
	       4008 => "DBI Execution of Statement Failed",
	       4009 => "DBI Error Occurred",
	       4018 => "Error opening file",
	       4022 => "Ledger Code required",
	       4023 => "Could not find Ledger Database",
	       4024 => "Could not find Ledger Name",
	       4025 => "Could not find Ledger Code",
	       4026 => "Ledger DB Name required",
	       4027 => "Ledger Name required",
	       4028 => "Database Handle required",
	     );

my %sql_statements;
%sql_statements = ( 
			1012 => "SELECT name, id FROM memorized_report_tb WHERE ledger_code = ? ORDER BY name, id",
			1015 => "SELECT DISTINCT(report_type_tb.id), report_type_tb.name, report_type_tb.editable FROM report_type_tb, report_data_tb WHERE report_type_tb.report_type = 'r' AND report_type_tb.id = report_data_tb.id ORDER BY report_type_tb.name, report_type_tb.id",
			1016 => "SELECT name FROM entity_tb WHERE database_name = ?",
			1017 => "SELECT code FROM ledger_tb WHERE name = ?",
			1018 => "SELECT color FROM ledger_tb WHERE code = ?",
			1022 => "SELECT DISTINCT(report_type_tb.id), report_type_tb.name, report_type_tb.editable FROM report_type_tb, report_data_tb WHERE report_type_tb.report_type = 'd' AND report_type_tb.id = report_data_tb.id ORDER BY report_type_tb.name, report_type_tb.id",
			1038 => "SELECT editable FROM report_type_tb WHERE id = ?",
			1067 => "SET datestyle TO 'POSTGRES,US'",
		     );

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";

    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";

    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
($db_name = $dbObj->dbName) =~ s/^(")([^"]+)(")$/$2/;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $db_type;
$db_type = "Pg";
my $dsn;
$dsn = "dbi:$db_type:dbname=$db_name;host=$db_host";

my $email_address;
$email_address = "james\@pcxperience.com";  # JavaScript error messages should be sent here.

%input = ();
cgi_lib::ReadParse(*input);  # gather the data from the client

use HTMLObject::Base;
use HTMLObject::ReadCookie;	# the module used to read cookies sent to us from the client.
use DBI;	# Using new Database engine for future compatibility.

# turn on debugging in Apache::DBI
#$Apache::DBI::DEBUG = 0;

my $cookies;
$cookies = HTMLObject::ReadCookie->new;

if (!exists $input{'command'})
{
  my_die("Error: Report.cgi - command missing", 4003, "Access Denied!");
}
else
{
  my $dbh;  # the handle of the database.
  # connect to the database.  The AutoCommit option will force transactions to be used.
  $dbh = DBI->connect($dsn, $db_user, $db_passwd, { RaiseError => 0, AutoCommit => 0 }) || my_db_die("DBI Error", 4004, "Database: $db_name\@$db_host");

  # force the database session to use Postgres/US convention date styles
  my $sth = $dbh->prepare( $sql_statements{1067} ) || my_db_die("Error: Report.cgi", 4007, "SQL Statement = '$sql_statements{1067}'");
  my $rc = $sth->execute() || my_db_die("Error: Report.cgi", 4008, "SQL Statement = '$sql_statements{1067}'");

  my $command = $input{'command'};
  if ($command eq "display_for_running")
  {
    display_for_running( dbh => $dbh );
  }
  elsif ($command eq "bottom")  # This is used by any of the frames that needs an empty bottom frame.
  {
    my $doc = HTMLObject::Base->new();
    $doc->setTitle("Bottom Frame");
    $doc->display();
  }
  else
  {
    my_die("Error: Report.cgi - Invalid command", 4001, "'$command' is invalid!");
  }

LEAVE_PROGRAM_EARLY:
  # disconnect from the database (if under mod_perl this will actually never happen. :)  )
  $dbh->disconnect || my_db_die ("DBI Error on Disconnect", 4009, "Database: $db_name\@$db_host");
}
exit;  # leave the program.

#****************************** Helper functions *****************************

# This function generates the display_for_running screen.
# Takes: error_string and/or status_string to display (optional)
sub display_for_running
{
  my %args = ( error_string => '', status_string => '', @_, );
  my $error_string = $args{'error_string'};
  my $status_string = $args{'status_string'};
  my $dbh = $args{'dbh'};

  # gather all reports from the report_type_tb and display them in a select box so the user can run them.  Also present's the memorized reports.
  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  # check to make sure we were given the ledger_db name
  my $ledger = $input{'ledger'};
  if (length $ledger == 0)
  {
    my_die("Error: Report.cgi - display_for_running", 4022, "The Ledger Code is required at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - get_ledger_color", 4028, "The DB Handle is required at line " . __LINE__);
  }

  my $doc = HTMLObject::Normal->new();
  $doc->setJavascriptErrorHandlerEmail(email => $email_address);
  $doc->setTitle("Select Report to Run");
  $doc->setFocus("body");

  my $date = `/bin/date +$date_format`;
  chomp $date;

  my $ledger_name = get_ledger_name( ledger_db => $ledger, dbh => $dbh );
  my $ledger_code = get_ledger_code( ledger_name => $ledger_name, dbh => $dbh );
  my $ledger_color = get_ledger_color( ledger_code => $ledger_code, dbh => $dbh );

  # create the header for the page.
  $doc->print(<<"END_OF_HEADER");
  <table border=0 cellpadding=0 cellspacing=0 width=100%>
    <tr>
      <td align=left width=47%>Working with Ledger <font color="$ledger_color">$ledger_name</font></td>
      <td align=left><a href="$site_dir/help.html#new_reporting_run" target="docs"><img border=0 src="$site_dir/images/help.jpg" alt="Help"></a></td>
      <td align=right>$date</td>
    </tr>
  </table>
  <hr>
  <table border=0 cellpadding=0 cellspacing=0 width=100%>
    <tr>
      <td align=left width=60% valign=top>
END_OF_HEADER

  if (length $error_string > 0)
  {
    $doc->print("<h2><font color=\"red\">Error</font>: &nbsp;$error_string</h2>\n");
  }
  if (length $status_string > 0)
  {
    $doc->print("<h2><font color=\"red\">Status</font>: &nbsp;$status_string</h2>\n");
  }

  # turn off the link decorations
  $doc->setLinkDecorations();

  $doc->print(<<"END_OF_BODY");
      </td>
      <td align=right width=40% nowrap valign=top>
        <form method=post action="$site_dir/cgi-bin/report.cgi" name="refresh_form" target="_top">
          <input type=hidden name="command" value="display_for_running">
          <input type=hidden name="ledger" value="$ledger">
          <input type=hidden name="open" value="open">
          <a href="" onMouseOver="window.status='Refresh'" onClick="document.refresh_form.action='$site_dir/cgi-bin/report.cgi';document.refresh_form.command.value='display_for_running';document.refresh_form.target='_top';document.refresh_form.submit();return false;">Refresh</a> | <a href="" onMouseOver="window.status='Report Maintenance'" onClick="document.refresh_form.action='$site_dir/cgi-bin/report_editor.cgi';document.refresh_form.command.value='display_for_edit';document.refresh_form.target='edit_report_window';window.open('', 'edit_report_window','resizable,scrollbars,status');document.refresh_form.submit();return false;">Report Maintenance</a>
        </form>
      </td>
    </tr>
  </table>

  <center>
  <h1>Select a Report to Run</h1>
  <form method=post name="report_form" action="$site_dir/cgi-bin/report_customization.cgi" onSubmit="return false;">
    <table border=0 cellpadding=0 cellspacing=0 bgcolor="$menu_bgcolor">
      <tr>
        <td align=right>
          <select name="report_group" onChange="report_group_change();">
            <option value="regular" selected>Standard
            <option value="memorized">Memorized
            <option value="detailed">Detail View
            <option value="payroll">Payroll
            <option value="bookkeeping">Other
          </select>
        </td>
        <td align=left>
          <select name="id" size=6 width=250 onChange="submit_form();">
          </select>
        </td>
      </tr>
    </table>
    <br>
    <input type=button name="run" value="Run" onClick="submit_form();">
  </form>
END_OF_BODY
  $doc->setFocus("javascript");

  $doc->print(<<"END_OF_CODE");

  function report_entry(value, text)
  {
    this.value = value;
    this.text = text;
  }

  var Site_Dir = "$site_dir";

  var reports = new Array();

END_OF_CODE
  # only gather the reports that are of type Regular to select from for running.
  my $sth = $dbh->prepare( $sql_statements{1015} ) || my_db_die("Error: Report.cgi - display_for_running", 4007, "SQL Statement = '$sql_statements{1015}'");
  my $rc = $sth->execute || my_db_die("Error: Report.cgi - display_for_running", 4008, "SQL Statement = '$sql_statements{1015}'");
  my $counter = 0;
  $doc->print("  reports[\"regular\"] = new Array();\n");
  while (my @info = $sth->fetchrow_array)
  {
    $doc->print("  reports[\"regular\"][" . $counter++ . "] = new report_entry(\"$info[0]\", \"$info[1]\");\n");
  }
  $sth->finish;

  # select memorized reports
  $sth = $dbh->prepare( $sql_statements{1012} ) || my_db_die("Error: Report.cgi - display_for_running", 4007, "SQL Statement = '$sql_statements{1012}'");
  $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - display_for_running", 4008, "SQL Statement = '$sql_statements{1012}', Ledger Code = '$ledger_code'");
  $counter = 0;
  $doc->print("  reports[\"memorized\"] = new Array();\n");
  while (my @info = $sth->fetchrow_array)
  {
    $doc->print("  reports[\"memorized\"][" . $counter++ . "] = new report_entry(\"$info[1]\", \"$info[0]\");\n");
  }
  $sth->finish;

  # select detailed view reports
  $sth = $dbh->prepare( $sql_statements{1022} ) || my_db_die("Error: Report.cgi - display_for_running", 4007, "SQL Statement = '$sql_statements{1022}'");
  $rc = $sth->execute || my_db_die("Error: Report.cgi - display_for_running", 4008, "SQL Statement = '$sql_statements{1022}'");
  $counter = 0;
  $doc->print("  reports[\"detailed\"] = new Array();\n");
  while (my @info = $sth->fetchrow_array)
  {
    $doc->print("  reports[\"detailed\"][" . $counter++ . "] = new report_entry(\"$info[0]\", \"$info[1]\");\n");
  }
  $sth->finish;


  # create the payroll and other entries

  $doc->print(<<"END_OF_CODE");
  reports["payroll"] = new Array();
  //reports["payroll"][0] = new report_entry("fica_and_medicare", "FICA and Medicare");
  //reports["payroll"][1] = new report_entry("payroll_record", "Payroll Record");
  reports["payroll"][0] = new report_entry("federal_941_quarterly_payroll", "Federal 941 Quarterly Payroll");
  reports["payroll"][1] = new report_entry("state_941_quarterly_payroll", "State 941 Quarterly Payroll");

  reports["bookkeeping"] = new Array();
  reports["bookkeeping"][0] = new report_entry("trial_balance", "Trial Balance");
  reports["bookkeeping"][1] = new report_entry("bank_reconciliation", "Bank Reconciliation");

  // Helper functions

  function report_group_change()
  {
    var d = document.report_form;
    var index = d.report_group.selectedIndex;

    if (index == -1)
    {
      alert("You must select a Report Group to work with!");
      return;
    }

    var reportgroup = d.report_group.options[index].value;

    // Clear the id select box.
    d.id.options.length = 0;

    // fill in the id select box with the selected groups reports
    for (var i=0; i < reports[reportgroup].length; i++)
    {
      d.id.options[d.id.length] = new Option(reports[reportgroup][i].text, reports[reportgroup][i].value);
    }
  }

  function validate_form()
  {
    // make sure that a report has been selected to run.
    if (document.report_form.id.selectedIndex > -1 && document.report_form.report_group.selectedIndex > -1)
    {
      return true;
    }
    else
    {
      alert("You must select a report to run!");
      return false;
    }
  }

  function submit_form()
  {
    var d = document.report_form;
    var w;

    if (validate_form())
    {
      var index = d.report_group.selectedIndex;
      var reportgroup = d.report_group.options[index].value;

      // Get the date.
      var date_obj = new Date();
      var milliseconds = date_obj.getTime().toString();
      var windowName = "report_" + milliseconds;

      // open the new window.
      w = window.open("", windowName, "resizable,scrollbars,status");

      var String = '<html>\\n';
      String += '<head><title>Run Report - '+d.id.options[d.id.selectedIndex].text+'</title></head>\\n';
      String += '<body bgcolor=white onLoad="document.run_report.submit();">\\n';
      if (reportgroup == "payroll" || reportgroup == "bookkeeping")
      {
        String += '<form name="run_report" method=post action="'+Site_Dir+'/cgi-bin/reports3.cgi">\\n';
        String += '<input type=hidden name="report" value="'+d.id.options[d.id.selectedIndex].value+'">\\n';
        String += '<input type=hidden name="open" value="open">\\n';
        String += '<input type=hidden name="customize" value="Select Report">\\n';
      }
      else
      {
        String += '<form name="run_report" method=post action="'+Site_Dir+'/cgi-bin/report_customization.cgi">\\n';
        String += '<input type=hidden name="command" value="customize_report">\\n';
        if (reportgroup == "memorized")
        {
          String += '<input type=hidden name="memorized" value="true">\\n';
          String += '<input type=hidden name="report_name" value="'+d.id.options[d.id.selectedIndex].text+'">\\n';
        }
        else
        {
          String += '<input type=hidden name="memorized" value="false">\\n';
        }
      }
      String += '<input type=hidden name="id" value="'+d.id.options[d.id.selectedIndex].value+'">\\n';
      String += '<input type=hidden name="ledger" value="$ledger">\\n';
      String += '<input type=hidden name="report_group" value="'+d.report_group[d.report_group.selectedIndex].value+'">\\n';
      String += '<input type="hidden" name="windowName" value="'+windowName+'">\\n';
      String += '<input type=submit name="customized" value="Customize Report"\\n';
      String += '</form>\\n';
      String += '</body>\\n';
      String += '</html>';

      w.document.open();
      w.document.writeln(String);
      w.document.close();

      d.id.selectedIndex = -1;
    }
  }

END_OF_CODE

  my $onload_code = "";
  $onload_code = "document.report_form.report_group.selectedIndex = 0;\n";
  $onload_code .= "report_group_change();\n";
  $doc->setOnload( code => $onload_code );

  my %ch = $cookies->getCookies(cookies => ['error_happened']);
  if (scalar keys %ch == 1)
  {
    # reset the cookie so that it is forgotten.
    my $date = `/bin/date --date "1 day ago" "+%a, %d-%b-%Y %T GMT"`;
    chomp $date;

    $doc->setCookie(name => 'error_happened', value => '', expires => "$date");
  }

  $doc->display();
}

# takes Title, Error Code, Custom Message
sub my_die
{
  my ($title, $code, $message) = @_;

  # close the database connection.
  #if (defined $dbh)
  #{
  #  $dbh->disconnect;
  #}

  my $doc = HTMLObject::Base->new();
  $doc->setErrorMessage(code => $code, message => $errors{$code});
  $doc->setError(code => $code);
  $doc->displayError(title => $title, message => "$message");
}

# takes Title, Error Code, Custom Message
sub my_db_die
{
  my ($title, $code, $message) = @_;
  my $error_str = $DBI::errstr;

  # close the database connection.
  #if (defined $dbh)
  #{
  #  $dbh->disconnect;
  #}

  my $doc = HTMLObject::Base->new();
  $doc->setErrorMessage(code => $code, message => $errors{$code});
  $doc->setError(code => $code);
  $doc->displayError(title => $title, message => "$message<br><br>\n$error_str");
}

###################### Data Gathering Functions converted from reports.pm to DBI structure #############################

# get_ledger_name
# Takes  : ledger_db name, handle to database connection
# Returns: ledger name
sub get_ledger_name
{
  my %args = ( @_, );
  my $ledger_db = $args{'ledger_db'};
  my $dbh = $args{'dbh'};
  my $ledger_name = "";

  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  # check to make sure we were given the ledger_db name and database handle
  if (length $ledger_db == 0)
  {
    my_die("Error: Report.cgi - get_ledger_name", 4026, "The Ledger DB Name is required at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - get_ledger_name", 4028, "The DB Handle is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1016} ) || my_db_die("Error: Report.cgi - get_ledger_name", 4007, "SQL Statement = '$sql_statements{1016}'");
  my $rc = $sth->execute($ledger_db) || my_db_die("Error: Report.cgi - get_ledger_name", 4008, "SQL Statement = '$sql_statements{1016}', Ledger DB = '$ledger_db'");
  my @info = $sth->fetchrow_array;

  if (!defined $info[0])
  {
    my_die("Error: Report.cgi - get_ledger_name", 4023, "Ledger DB = '$ledger_db'");
  }
  else
  {
    $ledger_name = $info[0];
  }
  $sth->finish;

  return $ledger_name;
}

# get_ledger_code
# Takes  : ledger_db name, handle to database connection
# Returns: ledger code
sub get_ledger_code
{
  my %args = ( @_, );
  my $ledger_name = $args{'ledger_name'};
  my $dbh = $args{'dbh'};
  my $ledger_code = "";

  # check to make sure we were given the ledger name and database handle
  if (length $ledger_name == 0)
  {
    my_die("Error: Report.cgi - get_ledger_code", 4027, "The Ledger Name is required at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - get_ledger_code", 4028, "The DB Handle is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1017} ) || my_db_die("Error: Report.cgi - get_ledger_code", 4007, "SQL Statement = '$sql_statements{1017}'");
  my $rc = $sth->execute($ledger_name) || my_db_die("Error: Report.cgi - get_ledger_code", 4008, "SQL Statement = '$sql_statements{1017}', Ledger Name = '$ledger_name'");
  my @info = $sth->fetchrow_array;

  if (!defined $info[0])
  {
    my_die("Error: Report.cgi - get_ledger_code", 4024, "Ledger Name = '$ledger_name'");
  }
  else
  {
    $ledger_code = $info[0];
  }
  $sth->finish;

  return $ledger_code;
}

# get_ledger_color
# Takes  : ledger_db name, handle to database connection
# Returns: ledger color
sub get_ledger_color
{
  my %args = ( @_, );
  my $ledger_code = $args{'ledger_code'};
  my $dbh = $args{'dbh'};
  my $ledger_color = "";

  # check to make sure we were given the ledger code and database handle
  if (length $ledger_code == 0)
  {
    my_die("Error: Report.cgi - get_ledger_color", 4022, "The Ledger Code is required at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - get_ledger_color", 4028, "The DB Handle is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1018} ) || my_db_die("Error: Report.cgi - get_ledger_color", 4007, "SQL Statement = '$sql_statements{1018}'");
  my $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - get_ledger_color", 4008, "SQL Statement = '$sql_statements{1018}', Ledger Code = '$ledger_code'");
  my @info = $sth->fetchrow_array;

  if (!defined $info[0])
  {
    my_die("Error: Report.cgi - get_ledger_color", 4025, "Ledger Name = '$ledger_code'");
  }
  else
  {
    $ledger_color = $info[0];
  }
  $sth->finish;

  return $ledger_color;
}
