#!/usr/bin/perl

# reports.cgi  -  Created by James Pattie, 07/20/1999
# This script presents the user with the main list of reports.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$ledger_db';
use vars '$ledger_name';
my $ledger_color;
my $ledger_code;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print "Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# now do a quick check and verify that we have either some beginning balance entries != 0.00 or some transactions.
my ($initial_balances_exist, $transactions_exist);
$initial_balances_exist = 0;
$transactions_exist = 0;

$result = $conn->exec("SELECT * FROM acct_bal_tb WHERE balance <> '0.00' AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Checking for initial account balances failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples > 0)
{
  $initial_balances_exist = 1;
}

$result = $conn->exec("SELECT ref_num FROM journal_entry_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Checking for transactions failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples > 0)
{
  $transactions_exist = 1;
}

if (!$transactions_exist && !$initial_balances_exist)
{
  PrintErrorHeader();
  print <<"END_OF_STYLE";
  <style type="text/css"><!--
    a:hover   { color: red }
    a:link    { text-decoration: none; color: blue }
    a:visited { text-decoration: none; color: blue }
    a:active  { text-decoration: none; color: blue }-->
  </style>
  <script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE

  print "<h1>Error:  There are no Initial Balances AND no Transactions in the Ledger!</h1>\n";
  # create a form that will present them with a Create New Transaction button or link to let them get some data into the system.
  print "<form name=\"create_transaction\" method=post action=\"$site_dir/cgi-bin/journal_activity.cgi\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<input type=hidden name=\"journal_code\" value=\"all_journals\">\n";
  print "<input type=hidden name=\"journal_bgcolor\" value=\"\">\n";
  print "<input type=hidden name=\"journal_command\" value=\"create\">\n";
  print "Create a <a href=\"$site_dir/cgi-bin/journal_activity.cgi\" onClick=\"go_here(document.create_transaction, '$site_dir/cgi-bin/journal_activity.cgi'); return false;\">New Transaction</a>\n";
  print "</form>\n";
  CleanUp();
}

print "<head><title>$prog_name $prog_version Reporting Main Menu</title></head>\n";
print "<body bgcolor=$body_bgcolor onLoad=\"do_load()\">\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
<script language=\"Javascript1.2\">
  <!--
  function create_report_group(value, text)
  {
    this.value = value;
    this.text = text;
  }

  function create_report_name(value, text)
  {
    this.value = value;
    this.text = text;
  }

  var report_group = new Array();
  var report_names = new Array();

  report_group[0] = new create_report_group(\"bookkeeping\", \"Bookkeeping\");
  report_group[1] = new create_report_group(\"payroll\", \"Payroll\");
  //report_group[2] = new create_report_group(\"asset\", \"Asset\");
  //report_group[3] = new create_report_group(\"investment\", \"Investment\");
  //report_group[4] = new create_report_group(\"analysis\", \"Analysis\");

  report_names[0] = new Array();
  report_names[1] = new Array();
  //report_names[2] = new Array();
  //report_names[3] = new Array();
  //report_names[4] = new Array();

  //report_names[0][0] = new create_report_name(\"income_statement\", \"Yearly Income Statement\");
  report_names[0][0] = new create_report_name(\"quarterly_income_statement\", \"Income Statement\");
  report_names[0][1] = new create_report_name(\"cash_flow\", \"Cash Flow\");
  report_names[0][2] = new create_report_name(\"balance_sheet\", \"Balance Sheet\");
  report_names[0][3] = new create_report_name(\"account_balance_sheet\", \"Accounts Balance Sheet\");
  report_names[0][4] = new create_report_name(\"account_activity_sheet\", \"Accounts Activity Sheet\");
  report_names[0][5] = new create_report_name(\"missing_checks\", \"Missing Checks - Coming Soon\");
  report_names[0][6] = new create_report_name(\"bank_reconciliation\", \"Bank Reconciliation\");
  report_names[0][7] = new create_report_name(\"trial_balance\", \"Trial Balance\");

  report_names[1][0] = new create_report_name(\"fica_and_medicare\", \"FICA and Medicare\");
  report_names[1][1] = new create_report_name(\"payroll_record\", \"Payroll Record\");
  report_names[1][2] = new create_report_name(\"federal_941_quarterly_payroll\", \"Federal 941 Quarterly Payroll\");
  report_names[1][3] = new create_report_name(\"state_941_quarterly_payroll\", \"State 941 Quarterly Payroll\");

  //report_names[2][0] = new create_report_name(\"asset_valuation\", \"Asset Valuation\");
  //report_names[2][1] = new create_report_name(\"depreciation\", \"Depreciation\");

  //report_names[3][0] = new create_report_name(\"1099_reconciliation\", \"1099 Reconciliation\");
  //report_names[3][1] = new create_report_name(\"investment_performance\", \"Investment Performance\");
  //report_names[3][2] = new create_report_name(\"capital_gains\", \"Capital Gains\");
  //report_names[3][3] = new create_report_name(\"investment_transactions\", \"Investment Transactions\");
  //report_names[3][4] = new create_report_name(\"price_comparison\", \"Price Comparison\");
  //report_names[3][5] = new create_report_name(\"portfolio_value\", \"Portfolio Value\");

  //report_names[4][0] = new create_report_name(\"quick_ratio\", \"Quick Ratio\");
  //report_names[4][1] = new create_report_name(\"current_ratio\", \"Current Ratio\");
  //report_names[4][2] = new create_report_name(\"inventory_turnover\", \"Inventory Turnover\");
  //report_names[4][3] = new create_report_name(\"accounts_receivable_turnover\", \"Accounts Receivable Turnover\");
  //report_names[4][4] = new create_report_name(\"days_sales_in_receivables\", \"Day's Sales in Receivables\");
  //report_names[4][5] = new create_report_name(\"debt_ratio\", \"Debt Ratio\");
  //report_names[4][6] = new create_report_name(\"times_interest_earned_ratio\", \"Times-Interest-Earned Ratio\");
  //report_names[4][7] = new create_report_name(\"return_on_net_sales\", \"Return on Net Sales\");
  //report_names[4][8] = new create_report_name(\"return_on_total_assets\", \"Return on Total Assets\");
  //report_names[4][9] = new create_report_name(\"return_on_common_stockholders_equity\", \"Return on Common Stockholder's Equity\");
  //report_names[4][10] = new create_report_name(\"eps_common_stock\", \"EPS Common Stock\");
  //report_names[4][11] = new create_report_name(\"pe\", \"PE\");
  //report_names[4][12] = new create_report_name(\"dividend_yield\", \"Dividend Yield\");
  //report_names[4][13] = new create_report_name(\"book_value\", \"Book Value\");

  function do_load()
  {
    document.report_select_form.report_group.selectedIndex = 0;
    report_group_change();
  }

  function report_group_change()
  {
    // clear the report_name select
    document.report_select_form.report.options.length = 0;
    // fill in select with the values from the array associated with the report group chosen
    for (var count=0; count < report_names[document.report_select_form.report_group.selectedIndex].length; count++)
    {
      document.report_select_form.report.options[count] = new Option(report_names[document.report_select_form.report_group.selectedIndex][count].text, report_names[document.report_select_form.report_group.selectedIndex][count].value);
    }
    document.report_select_form.report.selectedIndex = -1;
  }

  //-->
</script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%>Working with Ledger <font color=\"$ledger_color\">$ledger_name</font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#reporting\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Reporting Main Menu</h2>\n";

# create the form that lets them select from the main reports available.
my $temp;
$temp = $result->ntuples;
print <<"END_OF_FORM";
<form method=post name=\"report_select_form\" action=\"$site_dir/cgi-bin/reports3.cgi\">
<input type=hidden name=\"ledger\" value=\"$ledger_db\">
<table border=0 cellspacing=0 cellpadding=2 bgcolor=\"$menu_bgcolor\">
  <tr>
    <td align=right><font color=\"$menu_fgcolor\">Reports for</font></td>
    <td><select name=\"report_group\" onChange=\"report_group_change();\">
      <option value=\"bookkeeping\" selected>Bookkeeping
      <option value=\"payroll\">Payroll
    </select></td>
    <td><select name=\"report\" width=300 size=7 onChange=\"document.report_select_form.submit();\">
    </select></td>
  </tr>
</table>
<input type=hidden name=\"customize\" value=\"Select Report\">
<input type=submit name=\"submit_form\" value=\"Select Report\">
</form>

<form method=post name=\"report_form\" action=\"$site_dir/cgi-bin/report.cgi\" target=\"new_report_window\">
  <input type=hidden name=\"command\" value=\"display_for_edit\">
  <a href=\"\" onMouseOver=\"window.status='Work with new report style'\" onClick=\"document.report_form.submit(); return false;\">New Style of Reports (Edit & Create)</a>
</form>

<form method=post name=\"report_form2\" action=\"$site_dir/cgi-bin/report.cgi\" target=\"_top\">
  <input type=hidden name=\"command\" value=\"display_for_running\">
  <input type=hidden name=\"ledger\" value=\"$ledger_db\">
  <a href=\"\" onMouseOver=\"window.status='Run a report from the new report style'\" onClick=\"document.report_form2.submit(); return false;\">New Style of Reports (Run)</a>
</form>
END_OF_FORM
# these lines removed from the report_group select box after payroll.
#      <option value=\"asset\">Asset
#      <option value=\"investment\">Investment
#      <option value=\"analysis\">Analysis
# changed the submit button name from "open" to "customize" so the 3rd script would get the right input values

# print a horizontal rule to break the menu from the behind the scenes links
#print "<br><hr>\n";

print "</body>\n";
print "</html>\n";
exit 0;
