#!/usr/bin/perl
#$Id: reports3.cgi,v 1.1 2001/09/06 10:21:20 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# reports3.cgi  -  Created by James Pattie, 07/21/1999
# This script presents the customizable options for the report that the user selected.
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports; # the Reporting module

use vars '$date';
use vars '$count';
use vars '$ledger_db';
use vars '$ledger_name';
use vars '$report_group';
use vars '$group_name';
use vars '$report';
use vars '$report_name';
use vars '$accounts';
use vars '$known_years';
use vars '$working_year';
use vars '$report_obj';
use vars '$ledger_code';
use vars '$ledger_color';

my %BookKeepingFunctions;
%BookKeepingFunctions = ( "account_activity_sheet" => "Define",
			  "cash_flow" => "Define",
			  "balance_sheet" => "Define",
			  "account_balance_sheet" => "Define",
			  "income_statement" => "Define",
			  "quarterly_income_statement" => "Define",
			  "missing_checks" => "Define",
			  "bank_reconciliation" => "Define",
			  "trial_balance" => "Define"
);
my %PayrollFunctions;
%PayrollFunctions = ( "federal_941_quarterly_payroll" => "Define",
			  "state_941_quarterly_payroll" => "Define",
			  "fica_and_medicare" => "Define",
			  "payroll_record" => "Define"
);

my %JavaScriptFunctions;
%JavaScriptFunctions = ( "account_activity_sheet" => \&reports3AccountActivitySheetJavaScript,
			  "cash_flow" => "Define",
			  "balance_sheet" => "Define",
			  "account_balance_sheet" => \&reports3AccountBalanceSheetJavaScript,
			  "income_statement" => "Define",
			  "quarterly_income_statement" => \&reports3QuarterlyIncomeStatementJavaScript,
			  "missing_checks" => "Define",
			  "bank_reconciliation" => \&reports3BankReconciliationJavaScript,
			  "trial_balance" => "Define",
			  "federal_941_quarterly_payroll" => "Define",
			  "state_941_quarterly_payroll" => "Define",
			  "fica_and_medicare" => "Define",
			  "payroll_record" => "Define"
);

my %JavaScriptValidationFunctions;
%JavaScriptValidationFunctions = ( "account_activity_sheet" => \&reports3AccountActivitySheetJavaScriptValidation,
			  "cash_flow" => \&reports3CashFlowJavaScriptValidation,
			  "balance_sheet" => \&reports3BalanceSheetJavaScriptValidation,
			  "account_balance_sheet" => \&reports3AccountBalanceSheetJavaScriptValidation,
			  "income_statement" => "Define",
			  "quarterly_income_statement" => \&reports3QuarterlyIncomeStatementJavaScriptValidation,
			  "missing_checks" => "Define",
			  "bank_reconciliation" => \&reports3BankReconciliationJavaScriptValidation,
			  "trial_balance" => \&reports3TrialBalanceJavaScriptValidation,
			  "federal_941_quarterly_payroll" => \&reports3Federal941JavaScriptValidation,
			  "state_941_quarterly_payroll" => \&reports3State941JavaScriptValidation,
			  "fica_and_medicare" => "Define",
			  "payroll_record" => "Define"
);

my %JavaScriptValidationFunctionNames;
%JavaScriptValidationFunctionNames = ( "account_activity_sheet" => "reports3AccountActivitySheetJavaScriptValidation",
			  "cash_flow" => "reports3CashFlowJavaScriptValidation",
			  "balance_sheet" => "reports3BalanceSheetJavaScriptValidation",
			  "account_balance_sheet" => "reports3AccountBalanceSheetJavaScriptValidation",
			  "income_statement" => "Define",
			  "quarterly_income_statement" => "reports3QuarterlyIncomeStatementJavaScriptValidation",
			  "missing_checks" => "Define",
			  "bank_reconciliation" => "reports3BankReconciliationJavaScriptValidation",
			  "trial_balance" => "reports3TrialBalanceJavaScriptValidation",
			  "federal_941_quarterly_payroll" => "reports3Federal941JavaScriptValidation",
			  "state_941_quarterly_payroll" => "reports3State941JavaScriptValidation",
			  "fica_and_medicare" => "Define",
			  "payroll_record" => "Define"
);

my %CustomizedFormFunctions;
%CustomizedFormFunctions = ( "account_activity_sheet" => \&reports3AccountActivitySheetCustomizedFormItems,
			  "cash_flow" => \&reports3CashFlowCustomizedFormItems,
			  "balance_sheet" => \&reports3BalanceSheetCustomizedFormItems,
			  "account_balance_sheet" => \&reports3AccountBalanceSheetCustomizedFormItems,
			  "income_statement" => \&reports3IncomeStatementCustomizedFormItems,
			  "quarterly_income_statement" => \&reports3QuarterlyIncomeStatementCustomizedFormItems,
			  "missing_checks" => "Define",
			  "bank_reconciliation" => \&reports3BankReconciliationCustomizedFormItems,
			  "trial_balance" => \&reports3TrialBalanceCustomizedFormItems,
			  "federal_941_quarterly_payroll" => \&reports3Federal941CustomizedFormItems,
			  "state_941_quarterly_payroll" => \&reports3State941CustomizedFormItems,
			  "fica_and_medicare" => "Define",
			  "payroll_record" => "Define"
);

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# "import" the scripts that contain all the functions for actually doing the work of getting accounts,
# outputting JavaScript code, and printing HTML form code
require "./reports_account_balance_sheet.pm";
require "./reports_account_activity_sheet.pm";
use reports_accounts;
require "./reports_balance_sheet.pm";
require "./reports_cash_flow.pm";
require "./reports_helpers.pm";
require "./reports_income_statement.pm";
require "./reports_quarterly_income_statement.pm";
require "./reports_federal_941.pm";
require "./reports_state_941.pm";
require "./reports_trial_balance.pm";
require "./reports_bank_reconciliation.pm";

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

if (not exists $input{'customize'})
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Access!</h1>\n";
  print "$sys_admin";
  CleanUp();
}

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

$report_group = $input{'report_group'};
if (length $report_group == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must select a report group to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

if (not ($report_group eq "bookkeeping" || $report_group eq "payroll")) # || $report_group eq "asset" || $report_group eq "investment" || $report_group eq "analysis"))
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Report Group specified!</h1>\n";
  print "Value specified was '$report_group'.<br>$sys_admin";
  CleanUp();
}

$report = $input{'report'};
if (length $report == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a report to run!</h1>\n";
  print "$back_button";
  CleanUp();
}

if ($report_group eq "bookkeeping")
{
  if (not exists $BookKeepingFunctions{$report})
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid option for Bookkeeping!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  if ($report eq "income_statement")
  {
    $report_name = "Yearly Income Statement";
  }
  elsif ($report eq "quarterly_income_statement")
  {
    $report_name = "Income Statement";
  }
  elsif ($report eq "cash_flow")
  {
    $report_name = "Cash Flow";
  }
  elsif ($report eq "balance_sheet")
  {
    $report_name = "Balance Sheet";
  }
  elsif ($report eq "account_balance_sheet")
  {
    $report_name = "Accounts Balance Sheet";
  }
  elsif ($report eq "account_activity_sheet")
  {
    $report_name = "Accounts Activity Sheet";
  }
  elsif ($report eq "missing_checks")
  {
    $report_name = "Missing Checks";
  }
  elsif ($report eq "bank_reconciliation")
  {
    $report_name = "Bank Reconciliation";
  }
  elsif ($report eq "trial_balance")
  {
    $report_name = "Trial Balance";
  }
}
elsif ($report_group eq "payroll")
{
  if (not exists $PayrollFunctions{$report})
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid option for Payroll!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  if ($report eq "fica_and_medicare")
  {
    $report_name = "FICA and Medicare";
  }
  elsif ($report eq "payroll_record")
  {
    $report_name = "Payroll Record";
  }
  elsif ($report eq "federal_941_quarterly_payroll")
  {
    $report_name = "Federal 941 Quarterly Payroll";
  }
  elsif ($report eq "state_941_quarterly_payroll")
  {
    $report_name = "State 941 Quarterly Payroll";
  }
}
#elsif ($report_group eq "asset")
#{
#  if (not ($report eq "asset_valuation" || $report eq "depreciation"))
#  {
#    PrintErrorHeader();
#   print "$sys_admin";
#    CleanUp();
#  }
#  if ($report eq "asset_valuation")
#  {
# }
#  elsif ($report eq "depreciation")
#  {
#    $report_name = "Depreciation";
#  }
#}
#elsif ($report_group eq "investment")
#{
#  if (not ($report eq "1099_reconciliation" || $report eq "investment_performance" || $report eq "capital_gains" || $report eq "investment_transactions" || $report eq "price_comparison" || $report eq "portfolio_value"))
#  {
#    PrintErrorHeader();
#    print "<h1>Error:  Invalid option for Investment!</h1>\n";
#    print "$sys_admin";
#    CleanUp();
#  }
#  if ($report eq "1099_reconciliation")
#  {
#    $report_name = "1099 Reconciliation";
#  }
#  elsif ($report eq "investment_performance")
#  {
#    $report_name = "Investment Performance";
#  }
#  elsif ($report eq "capital_gains")
#  {
#    $report_name = "Capital Gains";
#  }
#  elsif ($report eq "investment_transactions")
#  {
#    $report_name = "Investment Transactions";
#  }
#  elsif ($report eq "price_comparison")
#  {
#    $report_name = "Price Comparison";
#  }
#  elsif ($report eq "portfolio_value")
#  {
#    $report_name = "Portfolio Value";
#  }
#}
#elsif ($report_group eq "analysis")
#{
#  if (not ($report eq "quick_ratio" || $report eq "current_ratio" || $report eq "inventory_turnover" || $report eq "accounts_receivable_turnover" || $report eq "days_sales_in_receivables" || $report eq "debt_ratio" || $report eq "times_interest_earned_ratio" || $report eq "return_on_net_sales" || $report eq "return_on_total_assets" || $report eq "return_on_common_stockholders_equity" || $report eq "eps_common_stock" || $report eq "pe" || $report eq "dividend_yield" || $report eq "book_value"))
#  {
#    PrintErrorHeader();
#    print "<h1>Error:  Invalid option for Analysis!</h1>\n";
#    print "$sys_admin";
#    CleanUp();
#  }
#  if ($report eq "quick_ratio")
#  {
#    $report_name = "Quick Ratio";
#  }
#  elsif ($report eq "current_ratio")
#  {
#    $report_name = "Current Ratio";
#  }
#  elsif ($report eq "inventory_turnover")
#  {
#    $report_name = "Inventory Turnover";
#  }
#  elsif ($report eq "accounts_receivable_turnover")
#  {
#    $report_name = "Accounts Receivable Turnover";
#  }
#  elsif ($report eq "days_sales_in_receivables")
#  {
#    $report_name = "Days Sales in Receivables";
#  }
#  elsif ($report eq "debt_ratio")
#  {
#    $report_name = "Debt Ratio";
#  }
#  elsif ($report eq "times_interest_earned_ratio")
#  {
#    $report_name = "Times-Interest-Earned Ratio";
#  }
#  elsif ($report eq "return_on_net_sales")
#  {
#    $report_name = "Return on Net Sales";
#  }
#  elsif ($report eq "return_on_total_assets")
#  {
#    $report_name = "Return on Total Assets";
#  }
#  elsif ($report eq "return_on_common_stockholders_equity")
#  {
#    $report_name = "Return on Common Stockholders Equity";
#  }
#  elsif ($report eq "eps_common_stock")
#  {
#    $report_name = "EPS Common Stock";
#  }
#  elsif ($report eq "pe")
#  {
# }
#  elsif ($report eq "dividend_yield")
#  {
#    $report_name = "Dividend Yield";
#  }
#  elsif ($report eq "book_value")
#  {
#    $report_name = "Book Value";
#  }
#}

if ($report_group eq "bookkeeping")
{
  $group_name = "Bookkeeping";
}
elsif ($report_group eq "payroll")
{
  $group_name = "Payroll";
}
#elsif ($report_group eq "asset")
#{
#  $group_name = "Asset";
#}
#elsif ($report_group eq "investment")
#{
#  $group_name = "Investment";
#}
#elsif ($report_group eq "analysis")
#{
#  $group_name = "Analysis";
#}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print "Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# now do a quick check and verify that we have either some beginning balance entries != 0.00 or some transactions.
my ($initial_balances_exist, $transactions_exist);
$initial_balances_exist = 0;
$transactions_exist = 0;

$result = $conn->exec("SELECT * FROM acct_bal_tb WHERE balance <> '0.00' AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Checking for initial account balances failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples > 0)
{
  $initial_balances_exist = 1;
}

$result = $conn->exec("SELECT ref_num FROM journal_entry_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Checking for transactions failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples > 0)
{
  $transactions_exist = 1;
}

if (!$transactions_exist && !$initial_balances_exist)
{
  PrintErrorHeader();
  print <<"END_OF_STYLE";
  <style type="text/css"><!--
    a:hover   { color: red }
    a:link    { text-decoration: none; color: blue }
    a:visited { text-decoration: none; color: blue }
    a:active  { text-decoration: none; color: blue }-->
  </style>
  <script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE

  print "<h1>Error:  There are no Initial Balances AND no Transactions in the Ledger!</h1>\n";
  # create a form that will present them with a Create New Transaction button or link to let them get some data into the system.
  print "<form name=\"create_transaction\" method=post action=\"$site_dir/cgi-bin/journal_activity.cgi\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<input type=hidden name=\"journal_code\" value=\"all_journals\">\n";
  print "<input type=hidden name=\"journal_bgcolor\" value=\"\">\n";
  print "<input type=hidden name=\"journal_command\" value=\"create\">\n";
  print "Create a <a href=\"$site_dir/cgi-bin/journal_activity.cgi\" onClick=\"go_here(document.create_transaction, '$site_dir/cgi-bin/journal_activity.cgi'); return false;\">New Transaction</a>\n";
  print "</form>\n";
  CleanUp();
}

# get the current working year
$working_year = $report_obj->GetCurrentWorkingYear();
if ($report_obj->didErrorOccur)
{
  PrintErrorHeader();
  print "<h1>Error:  Getting the Current Fiscal Year failed!</h1>\n";
  print $report_obj->getErrorMessage;
  CleanUp();
}

# lookup the accounts needed for this report, if any.
if ($report_group eq "bookkeeping")
{
  reports3GetBookkeepingAccounts();
}
elsif ($report_group eq "payroll")
{
}
#elsif ($report_group eq "asset")
#{
#}
#elsif ($report_group eq "investment")
#{
#}
#elsif ($report_group eq "analysis")
#{
#}

use vars '$OnLoadString';
$OnLoadString = "";
use vars '$onResetString';
$onResetString = "";
use vars qw( $name_Num_onclickString $Num_name_onclickString );
$name_Num_onclickString = $Num_name_onclickString = "";

print "<head><title>$prog_name $prog_version - Options for '$report_name' Report</title>\n";
if (exists $JavaScriptFunctions{$report} && $JavaScriptFunctions{$report} ne "Define") # call the JavaScript function for the current report.
{
  $JavaScriptFunctions{$report}->();
}
print "<script language=\"JavaScript1.2\">\n";
if (exists $JavaScriptValidationFunctions{$report} && $JavaScriptValidationFunctions{$report} ne "Define") # call the JavaScript Validation generation function in the report
{
  $JavaScriptValidationFunctions{$report}->();
}
print <<"END_OF_SCRIPT";
  function ValidateReportForm()
  {
    var msg = "";
    var doc = document.customize_form;
    var tmp = "";

END_OF_SCRIPT

if (exists $JavaScriptValidationFunctionNames{$report} && $JavaScriptValidationFunctionNames{$report} ne "Define")
{
  print "    tmp = " . $JavaScriptValidationFunctionNames{$report} . "(doc);\n";
}
else
{
  print "    msg += \"This report is not finished!  \\nPlease contact the Systems Administrator if you feel this is in error!\\n\";";
}

print <<"END_OF_SCRIPT";
    if (tmp.length > 0)
    {
      msg += tmp;
    }
    if (msg.length > 0)
    {
      alert(msg);
      return false;
    }
    return true;
  }
END_OF_SCRIPT
print "</script>\n";
print "</head>\n";
print "<body bgcolor=$body_bgcolor $OnLoadString>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;
my $time;
$time = `/bin/date +%s`;
chomp $time;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%>Working with Ledger <font color=\"blue\">$ledger_name</font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#Report_$report\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);
print "<h2>Customize <font color=\"red\">$report_name</font> Report</h2>\n";

print "<form name=\"customize_form\" method=post action=\"$site_dir/cgi-bin/reports4.cgi\" target=\"report_win_$report\" onSubmit=\"return ValidateReportForm();\">\n"; # removed the _$time part from target name.
print <<"END_OF_FORM";
<input type=hidden name=\"ledger\" value=\"$ledger_db\">
<input type=hidden name=\"report_group\" value=\"$report_group\">
<input type=hidden name=\"report\" value=\"$report\">
<table border=0 cellspacing=0 cellpadding=2 bgcolor=\"$menu_bgcolor\" width=50%>
<tr><td align=center><font color=\"$menu_fgcolor\">Report Name:&nbsp;</font><input type=text name=\"custom_report_name\" value=\"$report_name\"></td></tr>
END_OF_FORM
if ($report ne "bank_reconciliation" && $report ne "trial_balance")
{
  print "<tr><td align=center><font color=\"$menu_fgcolor\">Always Show Balances/Activities = \$0.00&nbsp;</font><input type=radio name=\"show_zero_balances\" value=\"YES\">&nbsp;Yes&nbsp;&nbsp;&nbsp;<input type=radio name=\"show_zero_balances\" value=\"NO\" checked>&nbsp;No</td></tr>\n";
}
else
{
  print "<input type=hidden name=\"show_zero_balances\" value=\"NO\">\n";
}
print <<"END_OF_FORM";
<tr><td align=center><font color=\"$menu_fgcolor\">Show Accounts as:&nbsp;</font><input type=radio name=\"show_accounts_as\" value=\"name-number\" checked$name_Num_onclickString>&nbsp;Name&nbsp;-&nbsp;Number&nbsp;&nbsp;&nbsp;<input type=radio name=\"show_accounts_as\" value=\"number-name\"$Num_name_onclickString>&nbsp;Number&nbsp;-&nbsp;Name</td></tr>
<tr><td align=center><font color=\"$menu_fgcolor\">Text Font Size = </font><select name=\"font_size_text\">
    <option value="+2">+2
    <option value="+1">+1
    <option value="+0">default
    <option value="-1" selected>-1
    <option value="-2">-2
    </select>&nbsp;&nbsp;&nbsp;<font color=\"$menu_fgcolor\">Data Font Size = </font><select name=\"font_size_data\">
    <option value="+2">+2
    <option value="+1">+1
    <option value="+0">default
    <option value="-1" selected>-1
    <option value="-2">-2
    </select></td></tr>
END_OF_FORM

if ($report eq "cash_flow" || $report eq "income_statement" || $report eq "quarterly_income_statement")
{
  print "<tr><td align=center><font color=\"$menu_fgcolor\">Display by Sub-Entities:&nbsp;</font><input type=radio name=\"show_by_sub_entities\" value=\"YES\">&nbsp;Yes&nbsp;&nbsp;&nbsp;<input type=radio name=\"show_by_sub_entities\" value=\"NO\" checked>&nbsp;No</td></tr>\n";
}
elsif ($report eq "federal_941_quarterly_payroll" || $report eq "state_941_quarterly_payroll")
{
  print "<input type=hidden name=\"show_by_sub_entities\" value=\"NO\">\n";
}
if ($report eq "quarterly_income_statement")
{
  print "<tr><td align=center><font color=\"$menu_fgcolor\">Show Non-Tax section:&nbsp;</font><input type=radio name=\"show_non_tax_section\" value=\"YES\" checked>&nbsp;Yes&nbsp;&nbsp;&nbsp;<input type=radio name=\"show_non_tax_section\" value=\"NO\">&nbsp;No</td></tr>\n";
}

# generate the form items needed to customize the selected report
if (exists $CustomizedFormFunctions{$report} && $CustomizedFormFunctions{$report} ne "Define")
{
  $CustomizedFormFunctions{$report}->();
}
#elsif ($report eq "asset_valuation")
#{
#}
#elsif ($report eq "depreciation")
#{
#}
#elsif ($report eq "1099_reconciliation")
#{
#}
#elsif ($report eq "investment_performance")
#{
#}
#elsif ($report eq "capital_gains")
#{
#}
#elsif ($report eq "investment_transactions")
#{
#}
#elsif ($report eq "price_comparison")
#{
#}
#elsif ($report eq "portfolio_value")
#{
#}
#elsif ($report eq "quick_ratio")
#{
#}
#elsif ($report eq "current_ratio")
#{
#}
#elsif ($report eq "inventory_turnover")
#{
#}
#elsif ($report eq "accounts_receivable_turnover")
#{
#}
#elsif ($report eq "days_sales_in_receivables")
#{
#}
#elsif ($report eq "debt_ratio")
#{
#}
#elsif ($report eq "times_interest_earned_ratio")
#{
#}
#elsif ($report eq "return_on_net_sales")
#{
#}
#elsif ($report eq "return_on_total_assets")
#{
#}
#elsif ($report eq "return_on_common_stockholders_equity")
#{
#}
#elsif ($report eq "eps_common_stock")
#{
#}
#elsif ($report eq "pe")
#{
#}
#elsif ($report eq "dividend_yield")
#{
#}
#elsif ($report eq "book_value")
#{
#}

if ($report ne "federal_941_quarterly_payroll" && $report ne "state_941_quarterly_payroll" && $report ne "account_balance_sheet" && $report ne "account_activity_sheet" && $report ne "trial_balance" && $report ne "bank_reconciliation")
{
print <<"END_OF_FORM";
  <tr><td align=center><font color=\"$menu_fgcolor\">Precision of Data</font>&nbsp;&nbsp;<select name=\"data_precision\">
  <option value=\"1\">1
  <option value=\"10\">10
  <option value=\"100\">100
  <option value=\"1000\">1,000
  <option value=\"10000\">10,000
  <option value=\"100000\">100,000
  <option value=\"1000000\">1,000,000
  </select></td></tr>
END_OF_FORM
}
else
{
  print "  <input type=hidden name=\"data_precision\" value=\"1\">\n";
}
print <<"END_OF_FORM";
</table>
<input type=submit name=\"run\" value=\"Run Report\">&nbsp;<input type=reset value=\"Reset\" $onResetString>
</form>
END_OF_FORM

# print a horizontal rule to break the menu from the behind the scenes links
#print "<br><hr>\n";

print "</body>\n";
print "</html>\n";
exit 0;