/*
 * File: DynEl.js
 * Include with: <SCRIPT SRC="DynEl.js"></SCRIPT>
 *
 *
 * This file defines the DynEl class, which provides a portable API
 * to many Dynamic HTML features.
#$Id: DynEl.js,v 1.1 2001/09/06 10:21:20 moreejt Exp $
#This code is released under the GNU General Public License.
#If you downloaded the package, you should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
*/

var myIsNav4, myIsIE4;
var myBrowserVersion;
// check for browser version; needs to be 4 or greater
myBrowserVersion = parseInt(window.navigator.appVersion.charAt(0));
// check for browser type
if (myBrowserVersion >= 4)
{
  if (window.navigator.appName.indexOf("Netscape") != -1)
  {
    myIsNav4 = true;
    myIsIE4 = false;
  }
  else if (window.navigator.appName.indexOf("Microsoft") != -1)
  {
    myIsIE4 = true;
    myIsNav4 = false;
  }
}

/*
 * This is the constructor function for DynEl objects.
 * The arguments are the following:
 * window: The Window object in which the dynamic element is to appear
 * id:     The HTML ID for the dynamic element.  Must be unique.
 * body:   HTML text that constitutes the body of the dynamic element
 * left:   The optional initial X-coordinate of the element
 * top:    The optional initial Y-coordinate of the element
 * width:  The optional width of the element
 *
 * This constructor outputs a style sheet into the current document.
 * This means that it can only be called from the <HEAD> of the document
 * before any text has been output for display.
 */
function DynEl(window, id, body, left, top, width)
{
  // check for browser version
  this.browserVersion = parseInt(window.navigator.appVersion.charAt(0));
  // check for browser type
  if (this.browserVersion >= 4)
  {
    if (window.navigator.appName.indexOf("Netscape") != -1)
    {
      this.isNav4 = true;
      this.isIE4 = false;
    }
    else if (window.navigator.appName.indexOf("Microsoft") != -1)
    {
      this.isIE4 = true;
      this.isNav4 = false;
    }
  }

  // Remember some arguments for later.
  this.window = window;
  this.id = id;
  this.body = body;

  //Output a CSS-P style sheet for this element.
  var d = window.document;
  d.writeln('<STYLE TYPE="text/css">');
  d.write('#' + id + ' {position:absolute;');
  if (left)
  {
    d.write('left:' + left + ';');
  }
  if (top)
  {
    d.write('top:' + top + ';');
  }
  if (width)
  {
    d.write('width:' + width + ';');
  }
  d.writeln('}');
  d.writeln('</STYLE>');
}

/*
 * Now we define a bunch of methods for the DynEl class.
 * We define each method to check if we are running in Navigator, and
 * or Internet Explorer.  In this way, we define a portable API to
 * the common DHTML functionality of the two browsers.
 */

/*
 * This function outputs the dynamic element itself into the document.
 * It must be called before any other methods of the DynEl object can
 * be used.
 */
DynEl.prototype.output = function()
{
  if (this.isNav4)
  {
    var d = this.window.document; // Shortcut variable: saves typing

    // Output the element within a <DIV> tag.  Specify the element id.
    d.writeln('<DIV ID="' + this.id + '">');
    d.writeln(this.body);
    d.writeln("</DIV>");

    // Now for convenience, save a reference to the Layer object
    // created by this dynamic element.
    this.layer = d[this.id];
  }
  else if (this.isIE4)
  {
    var d = this.window.document;  // Shortcut variable: saves typing

    // Output the element within a <DIV> tag.  Specify the element id.
    d.writeln('<DIV ID="' + this.id + '">');
    d.writeln(this.body);
    d.writeln("</DIV>");

    // Now, for convenience, save references to the <DIV> element
    // we've created, and to its associated Style element.
    // These will be used throughout the mthods that follow.
    this.element = d.all[this.id];
    this.style = this.element.style;
  }
}

// Here are methods for moving, hiding, stacking, and otherwise
// manipulating the dynamic element.
DynEl.prototype.moveTo = function(x, y)
{
  if (this.isNav4)
  {
    this.layer.moveTo(x, y);
  }
  else if (this.isIE4)
  {
    this.style.pixelLeft = x;
    this.style.pixelTop = y;
  }
}
DynEl.prototype.moveBy = function(x, y)
{
  if (this.isNav4)
  {
    this.layer.moveBy(x, y);
  }
  else if (this.isIE4)
  {
    this.style.pixelLeft += x;
    this.style.pixelTop += y;
  }
}
DynEl.prototype.show = function()
{
  if (this.isNav4)
  {
    this.layer.visibility = "show";
  }
  else if (this.isIE4)
  {
    this.style.visibility = "visible";
  }
}
DynEl.prototype.hide = function()
{
  if (this.isNav4)
  {
    this.layer.visibility = "hide";
  }
  else if (this.isIE4)
  {
    this.style.visibility = "hidden";
  }
}
DynEl.prototype.setStackingOrder = function(z)
{
  if (this.isNav4)
  {
    this.layer.zIndex = z;
  }
  else if (this.isIE4)
  {
    this.style.zIndex = z;
  }
}
DynEl.prototype.setBgColor = function(color)
{
  if (this.isNav4)
  {
    this.layer.bgColor = color;
  }
  else if (this.isIE4)
  {
    this.style.backgroundColor = color;
  }
}
DynEl.prototype.setBgImage = function(image)
{
  if (this.isNav4)
  {
    this.layer.background.src = image;
  }
  else if (this.isIE4)
  {
    this.style.backgroundImage = image;
  }
}

// These methods query the position, size, and other properties
// of the dynamic element.
DynEl.prototype.getX = function()
{
  if (this.isNav4)
  {
    return this.layer.left;
  }
  else if (this.isIE4)
  {
    return this.style.pixelLeft;
  }
}
DynEl.prototype.getY = function()
{
  if (this.isNav4)
  {
    return this.layer.top;
  }
  else if (this.isIE4)
  {
    return this.style.pixelTop;
  }
}
DynEl.prototype.getWidth = function()
{
  if (this.isNav4)
  {
    return this.layer.width;
  }
  else if (this.isIE4)
  {
    return this.style.width;
  }
}
DynEl.prototype.getHeight = function()
{
  if (this.isNav4)
  {
    return this.layer.height;
  }
  else if (this.isIE4)
  {
    return this.style.height;
  }
}
DynEl.prototype.getStackingOrder = function()
{
  if (this.isNav4)
  {
    return this.layer.zIndex;
  }
  else if (this.isIE4)
  {
    return this.style.zIndex;
  }
}
DynEl.prototype.isVisible = function()
{
  if (this.isNav4)
  {
    return this.layer.visibility == "show";
  }
  else if (this.isIE4)
  {
    return this.style.visibility == "visible";
  }
}

/*
 * This method allows us to dynamically change the contents of
 * the dynamic element.  The argument or arguments should be HTML
 * strings which become the new body of the element.
 */
DynEl.prototype.setBody = function()
{
  if (this.isNav4)
  {
    for (var i = 0; i < arguments.length; i++)
    {
      this.layer.document.writeln(arguments[i]);
    }
    this.layer.document.close();
  }
  else if (this.isIE4)
  {
    var body = "";
    for (var i = 0; i < arguments.length; i++)
    {
      body += arguments[i] + "\n";
    }
    this.element.innerHTML = body;
  }
}

/*
 * This method registers a handler for the named event on the
 * element.  The event name argument should be the name of an
 * event handler property, such as "onmousedown" or "onkeypress".
 * The handler is a function that takes whatever action is necessary.
 * Because Navigator and IE do not have compatible Event objects,
 * all event details are passed as arguments to the handler function.
 * When invoked, the handler will be passed the following nine arguments:
 *  1) A reference to the DynEl object
 *  2) A string containing the event type
 *  3) The X-coordinate of the mouse, relative to the DynEl
 *  4) The Y-coordinate of the mouse, relative to the DynEl
 *  5) The mouse button that was clicked (if any)
 *  6) The Unicode code of the key that was pressed (if any)
 *  7) A boolean specifying whether the Shift key was down
 *  8) A boolean specifying whether the Control key was down
 *  9) A boolean specifying whether the Alt key was down
 * Event handlers that are not interested in all these arguments do
 * not have to declare them all in their argument lists, of course.
 */
DynEl.prototype.addEventHandler = function(eventname, handler)
{
  if (this.isNav4)
  {
    // Arrange to capture events on this layer.
    this.layer.captureEvents(DynEl._eventmasks[eventname]);
    var dynel = this;  // Current DynEl for use in the nested function.
    // Define an event handler that will invoke the specified handler,
    // and pass it the nine arguments specified above.
    this.layer[eventname] = function(event)
    {
      return handler(dynel, event.type, event.x, event.y, event.which, event.which,
      		    ((event.modifiers & Event.SHIFT_MASK) != 0),
      		    ((event.modifiers & Event.CTRL_MASK) != 0),
      		    ((event.modifiers & Event.ALT_MASK) != 0));
    }
  }
  else if (this.isIE4)
  {
    var dynel = this;  // Current DynEl for use in the nested function
    // Set IE4 event handler that invokes the specified handler
    // with the appropriate nine arguments.
    this.element[eventname] = function()
    {
      var e = dynel.window.event;
      e.cancelBubble = true;
      return handler(dynel, e.type, e.x, e.y, e.button, e.keyCode, e.shiftKey, e.ctrlKey, e.altKey);
    }
  }
}

/*
 * This method unregisters the named event handler.  It should be
 * called with a single string argument such as "onmouseover".
 */
DynEl.prototype.removeEventHandler = function(eventname)
{
  if (this.isNav4)
  {
    this.layer.releaseEvents(DynEl._eventmasks[eventname]);
    delete this.layer[eventname];
  }
  else if (this.isIE4)
  {
    delete this.element[eventname];
  }
}

/*
 * This array is used internally by the two methods above to map
 * from event name to event type.
 */
if (myIsNav4)
{
  DynEl._eventmasks = {
	onabort:Event.ABORT, onblur:Event.BLUR, onchange:Event.CHANGE,
	onclick:Event.CLICK, ondblclick:Event.DBLCLICK,
	ondragdrop:Event.DRAGDROP, onerror:Event.ERROR,
	onfocus:Event.FOCUS, onkeydown:Event.KEYDOWN,
	onkeypress:Event.KEYPRESS, onkeyup:Event.KEYUP, onload:Event.LOAD,
	onmouseout:Event.MOUSEOUT, onmouseover:Event.MOUSEOVER,
	onmouseup:Event.MOUSEUP, onmove:Event.MOVE, onreset:Event.RESET,
	onresize:Event.RESIZE, onselect:Event.SELECT, onsubmit:Event.SUBMIT,
	onunload:Event.UNLOAD
  };
}