#! /usr/bin/perl
# This script will create the directories and copy the images
# to the images directory of the webserver as configured in
# Portal::Data::Config.  This is a wrapper that calls the
# /usr/lib/pcx_portal/install_app.pl script to do the work.

# install.pl - Created by James Pattie, (james@pcxperience.com)
# Copyright (c) 2000, PC & Web Xperience, Inc. (http://www.pcxperience.com/)
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.
# 07/02/2001

use Portal::Data::Config;
use strict;
use Portal::Language;
use Portal::Accounting;

$| = 1;

# These are the only customizable variables you need to mess with unless you are doing more work than the portal install_app.pl script is going to do for you.

my $appName = "Accounting";  # make sure this is Uppercase (first letter) at least.
my $configVersion = "0.1";  # The version of your Config.pm config file
my $override = "1";   # 1 = override/backup config file, 0 = don't override config file
my $DBVersion = "$Portal::Accounting::release.$Portal::Accounting::DBVersion";

print "DBversion: $DBVersion\n";

# end customizable variables

my $prefix = "";
if ($#ARGV == 0)
{
  $prefix = $ARGV[0];
  if (! -d $prefix)
  {
    die "Error!  Prefix directory = '$prefix' does not exist!\n";
  }
}

my $langObj = Portal::Language->new(lang => 'en');
if ($langObj->didErrorOccur)
{
  die($langObj->errorMessage);
}

my $configObj = undef;
eval { $configObj = Portal::Data::Config->new(langObj => $langObj); };
if ($@)
{
  die "Error instantiating Portal::Data::Config->new()!  Error = $@\n";
}
if ($configObj->didErrorOccur)
{
  die $configObj->errorMessage;
}

my $dir = $prefix . $configObj->webRoot . $configObj->siteDir;
my $DBdir = "$dir/install/Accounting/db/$DBVersion";
my $DBdir2 = "$dir/install/Accounting/db";
my $ext = "psql";

#this doesn't seem to work here so I put it in MakeFile.PL too
#update the database file
system("/bin/cat $DBdir/schema-functions.$ext $DBdir/data.$ext $DBdir/system_reports.$ext > $DBdir2/create_db.$ext");

# run the /usr/lib/pcx_portal/install_app.pl script to actually do the work
# The portal must be installed on the machine that you are running this on
# especially if you are building an rpm!

my $location = `pwd`; chomp $location;
print "Running install_app.pl\n";
my $command = "/usr/lib/pcx_portal/install_app.pl --appname $appName --location $location --override $override --configversion $configVersion" . (length $prefix > 0 ? " --prefix $prefix" : "");
system($command) == 0 or die "Error running install_app.pl from Portal!  Error = $!\n";

# If something else needs to be installed do it now.

# see if they want to install the support files.
print "\nDo you want to install the CGI Support Files (needed to actually run the Accounting App)?  [Y]:";
my $input = "";
while (length $input == 0)
{
  $input = <STDIN>;
  chomp $input;
  if (length $input == 0)
  {
    $input = "y";
  }
}

if ($input =~ /^y$/i)
{
  my @args;

  # remove old files.
  @args = ("rm", "-rf", "$dir/js/$appName/accounting");
  system(@args) == 0 or print "Warning! Unable to remove old files from '$dir/js/$appName/accounting'!  Error = $!\n";

  system("cp -rf files/accounting $dir/js/$appName/") == 0 or die "Error! Could not install CGI support scripts to '$dir/js/$appName/accounting'!  Error = $!\n";
  print "Setting permissions on support files:...\n";
  system("chmod -R a+r $dir/js/$appName/accounting") == 0 or print "Warning!  Could not set world read permissions on '$dir/js/$appName/accounting'\n";
  system("chmod a+x $dir/js/$appName/accounting/cgi-bin/*.cgi") == 0 or print "Warning!  Could not set world read permissions on '$dir/js/$appName/accounting/cgi-bin/*.cgi'\n";
}
else
{
  print "\nSkipping Support Files setup...  (This could be dangerous!)\n";
}

print "\nInstallation is done.  Goto " . $configObj->httpType . "://" . $configObj->myHostName . $configObj->siteDir . "/ to use the Portal.\n" if (length $prefix == 0);
print "\nDon't forget to re-run setup.pl to finish configuring $appName in the Portal.\n" if (length $prefix > 0);
